# Data sources:
# Yoruba names: https://en.wikipedia.org/wiki/List_of_Yoruba_given_names
# Igbo names: https://en.wikipedia.org/wiki/Igbo_names
# Hausa names: https://en.wikipedia.org/wiki/Hausa_names
# Nigerian English names: https://en.wikipedia.org/wiki/Nigerian_name
# Additional references:
# - Behind the Name (Igbo, Yoruba): https://www.behindthename.com
# - Journal of West African Languages (Hausa naming practices, 2016)

from randum.providers.person import Provider as PersonProvider


class Provider(PersonProvider):
    # Male first names
    first_names_male = [
        "John",
        "Emmanuel",
        "Peter",
        "Samuel",
        "David",
        "Michael",
        "Joseph",
        "Daniel",
        "James",
        "Paul",
        "Gabriel",
        "Joshua",
        "Philip",
        "Andrew",
        "Stephen",
        "Benjamin",
        "Mark",
        "Nathaniel",
        "Simon",
        "Cornelius",
    ]

    # Female first names
    first_names_female = [
        "Mary",
        "Grace",
        "Joy",
        "Patience",
        "Elizabeth",
        "Victoria",
        "Sarah",
        "Deborah",
        "Esther",
        "Blessing",
        "Charity",
        "Hope",
        "Gloria",
        "Agnes",
        "Peace",
        "Comfort",
        "Juliet",
        "Ruth",
        "Angela",
        "Faith",
    ]

    # Combined list
    first_names = first_names_male + first_names_female

    # Prefixes
    prefixes_male = ["Mr.", "Chief", "Dr.", "Prof.", "Engr."]
    prefixes_female = ["Mrs.", "Miss", "Dr.", "Prof.", "Lady"]

    prefixes = prefixes_male + prefixes_female

    # Last names
    last_names = [
        "Okonkwo",
        "Adeyemi",
        "Olawale",
        "Chukwu",
        "Eze",
        "Obi",
        "Abiola",
        "Okafor",
        "Balogun",
        "Uche",
        "Ogunleye",
        "Nnamani",
        "Adetokunbo",
        "Ojo",
        "Ekwueme",
        "Oshodi",
        "Ibrahim",
        "Akinwale",
        "Obasanjo",
        "Oyekan",
    ]



from .. import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for `nl_BE` locale.

    Information about the Belgian banks can be found on the website
    of the National Bank of Belgium:
    https://www.nbb.be/nl/betalingen-en-effecten/betalingsstandaarden/bankidentificatiecodes
    """

    bban_format = "############"
    country_code = "BE"

    banks = (
        "Argenta Spaarbank",
        "AXA Bank",
        "Belfius Bank",
        "BNP Paribas Fortis",
        "Bpost Bank",
        "Crelan",
        "Deutsche Bank AG",
        "ING België",
        "KBC Bank",
    )
    swift_bank_codes = (
        "ARSP",
        "AXAB",
        "BBRU",
        "BPOT",
        "DEUT",
        "GEBA",
        "GKCC",
        "KRED",
        "NICA",
    )
    swift_location_codes = (
        "BE",
        "B2",
        "99",
        "21",
        "91",
        "23",
        "3X",
        "75",
        "2X",
        "22",
        "88",
        "B1",
        "BX",
        "BB",
    )
    swift_branch_codes = [
        "203",
        "BTB",
        "CIC",
        "HCC",
        "IDJ",
        "IPC",
        "MDC",
        "RET",
        "VOD",
        "XXX",
    ]

    def bban(self) -> str:
        """Generate a valid BBAN."""
        account_number = self._generate_account_number()
        check_digits = self._calculate_mod97(account_number)
        return f"{account_number}{check_digits}"

    def iban(self) -> str:
        """Generate a valid IBAN."""
        bban = self.bban()
        iban_check_digits = self._calculate_iban_check_digits(bban)
        return f"{self.country_code}{iban_check_digits}{bban}"

    def _generate_account_number(self) -> str:
        """Generate a random 10-digit account number."""
        return self.numerify("##########")

    def _calculate_mod97(self, account_number: str) -> str:
        """Calculate the mod 97 check digits for a given account number."""
        remainder = int(account_number) % 97
        return str(remainder).zfill(2) if remainder != 0 else "97"

    def _calculate_iban_check_digits(self, bban: str) -> str:
        """Calculate the IBAN check digits using mod 97 algorithm."""
        raw_iban = f"{bban}{self.country_code}00"
        numeric_iban = "".join(str(ord(char) - 55) if char.isalpha() else char for char in raw_iban)
        check_digits = 98 - (int(numeric_iban) % 97)
        return str(check_digits).zfill(2)



from typing import List

from randum.utils.decorators import lowercase, slugify_unicode

from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ("com", "net", "cl", "cl")
    tlds = ("com", "com", "com", "net", "org", "cl", "cl", "cl")
    replacements = (
        ("à", "a"),
        ("â", "a"),
        ("ã", "a"),
        ("á", "a"),
        ("ç", "c"),
        ("é", "e"),
        ("ê", "e"),
        ("í", "i"),
        ("ô", "o"),
        ("ö", "o"),
        ("õ", "o"),
        ("ó", "o"),
        ("ú", "u"),
    )

    @lowercase
    @slugify_unicode
    def domain_word(self) -> str:
        company: str = self.generator.format("company")
        company_elements: List[str] = company.split(" ")
        # select 2 items as companies include prefix
        name_items = company_elements[:2]
        return self._to_ascii("".join(name_items))



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    # Source: https://ismlar.com/top100

    formats_female = (
        "{{first_name_female}} {{last_name_female}}",
        "{{last_name_female}} {{first_name_female}}",
    )

    formats_male = (
        "{{first_name_male}} {{last_name_male}}",
        "{{last_name_male}} {{first_name_male}}",
    )

    formats = formats_male + formats_female

    first_names_female = (
        "Sa’diya",
        "Sumayya",
        "Safiya",
        "Yasmina",
        "Hadicha",
        "Samiya",
        "Maryam",
        "Asma",
        "Mursalina",
        "Mubina",
        "Rayyona",
        "Safina",
        "Madina",
        "Oysha",
        "Yasina",
        "Aylin",
        "Musfira",
        "Sabina",
        "Muslima",
        "Osiyo",
        "Soliha",
        "Imona",
        "Farzona",
        "Humayra",
        "Samira",
        "Maftuna",
        "Ifora",
        "Farangiz",
        "Sakina",
        "Robiya",
        "Elif",
        "Bibisora",
        "Omina",
        "Asmira",
        "Marjona",
        "Sabrina",
        "Zahro",
        "Malak",
        "Sevinch",
        "Afruza",
        "Sadia",
        "Jasmina",
        "Mushtariy",
        "Nuriya",
        "Muzayyana",
        "Anisa",
        "Fotima",
        "Diyora",
        "Ruhshona",
        "Mehrimoh",
        "Shahina",
        "Muxlisa",
        "Shukrona",
        "Milana",
        "Dilnur",
        "Iymona",
        "Mohinur",
        "Sarvinoz",
        "Shahnoza",
        "Alzina",
        "Munisa",
        "Charos",
        "Aliya",
        "Odina",
        "Durdona",
        "Malika",
        "E’zoza",
        "Rayhona",
        "Roziya",
        "Samina",
        "Feruza",
        "Asila",
        "Fariza",
        "Muzdalifa",
        "Hadija",
        "Mohlaroyim",
        "Laylo",
        "Rumaysa",
        "Kumush",
        "Nilufar",
        "Saida",
        "Bonu",
        "Zarina",
        "Aziza",
        "Ansora",
        "Muizza",
        "Dilnoz",
        "Amira",
        "Dildora",
        "Ruqiya",
        "Dunyo",
        "Zinnura",
        "Nozima",
        "Zaynab",
        "Muhsina",
        "Habiba",
        "Sevara",
        "Amina",
        "Zakiyaxon",
        "Mumtoz",
    )

    first_names_male = (
        "Zubayr",
        "Mustafo",
        "Muhammad",
        "Umar",
        "Ali",
        "Imron",
        "Muhammadyusuf",
        "Ayub",
        "Abubakr",
        "Muhammadali",
        "Muhammadamin",
        "Usmon",
        "Ibrohim",
        "Bilol",
        "Zayd",
        "Muhammadyasin",
        "Yasin",
        "Aziz",
        "Samir",
        "Firdavs",
        "Abdulloh",
        "Behruz",
        "Shohrux",
        "Amir",
        "Yahyo",
        "Asad",
        "Kamron",
        "Mironshoh",
        "Islom",
        "Halid",
        "Habib",
        "Jamshid",
        "Shahzoda",
        "Akbar",
        "Abduaziz",
        "Sardor",
        "Javohir",
        "Abdurahmon",
        "Ja’far",
        "Muhammadziyo",
        "Muhammadumar",
        "Muhammadayub",
        "Akobir",
        "Jahongir",
        "Yusuf",
        "Diyor",
        "Salohiddin",
        "Farhod",
        "Humoyun",
        "Bobur",
        "Abror",
        "Samandar",
        "Asilbek",
        "Sarvar",
        "Muhammadrizo",
        "Shohjahon",
        "Ahmad",
        "Yunus",
        "Sanjar",
        "Doniyor",
        "Daler",
        "Shahzod",
        "Said",
        "Muhammadsafo",
        "Ismoil",
        "Nurmuhammad",
        "Muhammadamir",
        "Otabek",
        "Ulug‘bek",
        "Miron",
        "Xolid",
        "Dovud",
        "Damir",
        "Abbos",
        "Gabriel",
        "Zakariyo",
        "Abdulhamid",
        "Jalol",
        "Mahmud",
        "Shahriyor",
        "Saddam",
        "Fuzayl",
        "Akmal",
        "Alinur",
        "Hasan",
        "Murod",
        "Isfandiyor",
        "Miran",
        "Temir",
        "Muhammadaziz",
        "Abdumalik",
        "Muhammadsodiq",
        "Muslim",
        "Abduboriy",
        "Laziz",
        "Fariz",
        "Jasur",
        "Muhammadsolih",
        "Mahdi",
        "Anvar",
    )

    first_names = first_names_male + first_names_female

    last_names_female = (
        "Sadiyeva",
        "Sumayyeva",
        "Safiyeva",
        "Yasminova",
        "Hadichayeva",
        "Samiyeva",
        "Maryamova",
        "Asmanova",
        "Mursalinaeva",
        "Mubinova",
        "Rayyonova",
        "Safinayeva",
        "Madinaeva",
        "Oyshayeva",
        "Yasinaeva",
        "Aylinova",
        "Musfirayeva",
        "Sabinova",
        "Muslimaeva",
        "Osiyeva",
        "Solihayeva",
        "Imonova",
        "Farzonova",
        "Humayrova",
        "Samirova",
        "Maftunova",
        "Iforayeva",
        "Farangizova",
        "Sakinayeva",
        "Robiyayeva",
        "Elifova",
        "Bibisorayeva",
        "Ominova",
        "Asmiraeva",
        "Marjonova",
        "Sabrinova",
        "Zahroyeva",
        "Malakova",
        "Sevinchova",
        "Afruzova",
        "Sadiyeva",
        "Jasminova",
        "Mushtariyeva",
        "Nuriyeva",
        "Muzayyanova",
        "Anisayeva",
        "Fotimayeva",
        "Diyorayeva",
        "Ruhshonova",
        "Mehrimohova",
        "Shahinova",
        "Muxlisayeva",
        "Shukronova",
        "Milanova",
        "Dilnurova",
        "Iymonova",
        "Mohinurova",
        "Sarvinozova",
        "Shahnozova",
        "Alzinova",
        "Munisayeva",
        "Charosova",
        "Aliyeva",
        "Odinaeva",
        "Durdonova",
        "Malikova",
        "E’zozova",
        "Rayhonova",
        "Roziyayeva",
        "Saminova",
        "Feruzova",
        "Asilayeva",
        "Farizayeva",
        "Muzdalifayeva",
        "Hadijayeva",
        "Mohlaroyimova",
        "Laylova",
        "Rumaysayeva",
        "Kumushova",
        "Nilufarova",
        "Saidova",
        "Bonueva",
        "Zarinova",
        "Azizova",
        "Ansorova",
        "Muizzova",
        "Dilnozova",
        "Amirayeva",
        "Dildorova",
        "Ruqiyayeva",
        "Dunyoeva",
        "Zinnurova",
        "Nozimova",
        "Zaynabova",
        "Muhsinova",
        "Habibova",
        "Sevarova",
        "Aminova",
        "Zakiyaxonova",
        "Mumtozova",
    )

    last_names_male = (
        "Zubayrov",
        "Mustafayev",
        "Muhammadov",
        "Umarov",
        "Aliyev",
        "Imronov",
        "Muhammadyusupov",
        "Ayubov",
        "Abubakrov",
        "Muhammadaliyev",
        "Muhammadaminov",
        "Usmonov",
        "Ibrohimov",
        "Bilolov",
        "Zayidov",
        "Muhammadyasinov",
        "Yasinov",
        "Azizov",
        "Samirov",
        "Firdavsev",
        "Abdullohov",
        "Behruzov",
        "Shohruxov",
        "Amirov",
        "Yahyoev",
        "Asadov",
        "Kamronov",
        "Mironshohov",
        "Islomov",
        "Halidov",
        "Habibov",
        "Jamshidov",
        "Shahzodov",
        "Akbarov",
        "Abduazizov",
        "Sardorov",
        "Javohirov",
        "Abdurahmonov",
        "Ja’farov",
        "Muhammadziyoev",
        "Muhammadumarov",
        "Muhammadayubov",
        "Akobirov",
        "Jahongirov",
        "Yusupov",
        "Diyorov",
        "Salohiddinov",
        "Farhodov",
        "Humoyunov",
        "Boburov",
        "Abrorov",
        "Samandarov",
        "Asilbekov",
        "Sarvarov",
        "Muhammadrizoyev",
        "Shohjahonov",
        "Ahmadov",
        "Yunusov",
        "Sanjarov",
        "Doniyorov",
        "Dalerov",
        "Shahzodov",
        "Saidov",
        "Muhammadsafoev",
        "Ismoilov",
        "Nurmuhammadov",
        "Muhammadamirov",
        "Otabekov",
        "Ulug‘bekov",
        "Mironov",
        "Xolidov",
        "Dovudov",
        "Damirov",
        "Abbosov",
        "Gabriyelov",
        "Zakariyoev",
        "Abdulhamidov",
        "Jalolov",
        "Mahmudov",
        "Shahriyorov",
        "Saddamov",
        "Fuzaylov",
        "Akmalov",
        "Alinurov",
        "Hasanov",
        "Murodov",
        "Isfandiyorov",
        "Miranov",
        "Temirov",
        "Muhammadazizov",
        "Abdumalikov",
        "Muhammadsodiqov",
        "Muslimov",
        "Abduboriyov",
        "Lazizov",
        "Farizov",
        "Mehrojov",
        "Muhammadsolihov",
        "Mahdiev",
        "Anvarov",
    )

    last_names = last_names_female + last_names_male



from .. import BaseProvider

localized = False


class Provider(BaseProvider):
    """Provider for vehicle and automotive-related data."""

    # Vehicle types
    vehicle_types = [
        "Sedan", "SUV", "Truck", "Coupe", "Convertible", "Hatchback",
        "Wagon", "Minivan", "Van", "Pickup Truck", "Sports Car",
        "Luxury Car", "Compact Car", "Crossover", "Roadster",
        "Limousine", "Bus", "Motorcycle", "Scooter", "ATV",
        "RV", "Camper", "Trailer", "Semi-Truck", "Tow Truck"
    ]

    # Car manufacturers
    car_makes = [
        "Toyota", "Honda", "Ford", "Chevrolet", "Nissan", "BMW", "Mercedes-Benz",
        "Audi", "Volkswagen", "Hyundai", "Kia", "Mazda", "Subaru", "Lexus",
        "Jeep", "Ram", "GMC", "Dodge", "Chrysler", "Buick", "Cadillac",
        "Tesla", "Volvo", "Porsche", "Jaguar", "Land Rover", "Ferrari",
        "Lamborghini", "Maserati", "Bentley", "Rolls-Royce", "Aston Martin",
        "McLaren", "Bugatti", "Alfa Romeo", "Fiat", "Peugeot", "Renault",
        "Citroën", "Mini", "Mitsubishi", "Infiniti", "Acura", "Genesis"
    ]

    # Popular car models by make
    car_models = {
        "Toyota": ["Camry", "Corolla", "RAV4", "Highlander", "Tacoma", "Tundra", "Prius", "4Runner"],
        "Honda": ["Civic", "Accord", "CR-V", "Pilot", "Odyssey", "Fit", "HR-V", "Ridgeline"],
        "Ford": ["F-150", "Mustang", "Explorer", "Escape", "Edge", "Fusion", "Ranger", "Bronco"],
        "Chevrolet": ["Silverado", "Equinox", "Malibu", "Traverse", "Tahoe", "Camaro", "Corvette", "Blazer"],
        "Nissan": ["Altima", "Sentra", "Rogue", "Pathfinder", "Frontier", "Titan", "Maxima", "Murano"],
        "BMW": ["3 Series", "5 Series", "7 Series", "X3", "X5", "X7", "M3", "M5"],
        "Mercedes-Benz": ["C-Class", "E-Class", "S-Class", "GLC", "GLE", "GLS", "A-Class", "CLA"],
        "Audi": ["A3", "A4", "A6", "A8", "Q3", "Q5", "Q7", "Q8"],
        "Tesla": ["Model S", "Model 3", "Model X", "Model Y", "Cybertruck", "Roadster"],
        "default": ["Sedan", "SUV", "Coupe", "Hatchback", "Wagon"]
    }

    # Vehicle colors
    vehicle_colors = [
        "White", "Black", "Silver", "Gray", "Red", "Blue", "Green", "Yellow",
        "Orange", "Brown", "Beige", "Gold", "Purple", "Pink", "Burgundy",
        "Navy Blue", "Dark Green", "Light Blue", "Charcoal", "Pearl White",
        "Metallic Silver", "Midnight Black", "Racing Red", "Electric Blue"
    ]

    # Fuel types
    fuel_types = [
        "Gasoline", "Diesel", "Electric", "Hybrid", "Plug-in Hybrid",
        "Ethanol", "Biodiesel", "Natural Gas", "Propane", "Hydrogen"
    ]

    # Transmission types
    transmission_types = [
        "Automatic", "Manual", "CVT", "Semi-Automatic", "Dual-Clutch",
        "6-Speed Manual", "8-Speed Automatic", "10-Speed Automatic"
    ]

    # Engine sizes
    engine_sizes = [
        "1.5L", "1.6L", "1.8L", "2.0L", "2.4L", "2.5L", "3.0L", "3.5L",
        "4.0L", "5.0L", "5.7L", "6.2L", "6.4L", "V6", "V8", "V12",
        "Inline-4", "Inline-6", "Flat-4", "Flat-6"
    ]

    # Drive types
    drive_types = [
        "FWD", "RWD", "AWD", "4WD", "Front-Wheel Drive", "Rear-Wheel Drive",
        "All-Wheel Drive", "Four-Wheel Drive"
    ]

    # Vehicle features
    vehicle_features = [
        "Sunroof", "Moonroof", "Leather Seats", "Heated Seats", "Cooled Seats",
        "Navigation System", "Backup Camera", "Blind Spot Monitor",
        "Lane Departure Warning", "Adaptive Cruise Control", "Parking Sensors",
        "Keyless Entry", "Push Button Start", "Remote Start", "Apple CarPlay",
        "Android Auto", "Premium Sound System", "Bluetooth", "USB Ports",
        "Third Row Seating", "Roof Rack", "Tow Package", "Sport Package"
    ]

    # License plate patterns (US style)
    license_plate_patterns = [
        "###-####",
        "###-###",
        "ABC-####",
        "ABC-###",
        "##-ABC-##",
        "ABC-##-##",
    ]

    # VIN pattern (simplified)
    vin_pattern = "1HGBH41JXMN######"

    # Vehicle conditions
    vehicle_conditions = [
        "New", "Excellent", "Very Good", "Good", "Fair", "Poor",
        "Certified Pre-Owned", "Like New", "Salvage", "Rebuilt"
    ]

    # Dealership types
    dealership_types = [
        "New Car Dealership", "Used Car Dealership", "Luxury Dealership",
        "Franchise Dealership", "Independent Dealership", "Online Dealership",
        "Buy Here Pay Here", "Certified Pre-Owned Dealer"
    ]

    # Insurance types
    insurance_types = [
        "Liability", "Collision", "Comprehensive", "Full Coverage",
        "Uninsured Motorist", "Personal Injury Protection", "Gap Insurance",
        "Rental Reimbursement", "Roadside Assistance"
    ]

    def vehicle(self) -> str:
        """
        Generate a random vehicle type.
        
        :example: 'SUV'
        """
        return self.random_element(self.vehicle_types)

    def vehicle_type(self) -> str:
        """
        Generate a random vehicle type (alias for vehicle).
        
        :example: 'Sedan'
        """
        return self.vehicle()

    def car_make(self) -> str:
        """
        Generate a random car manufacturer.
        
        :example: 'Toyota'
        """
        return self.random_element(self.car_makes)

    def car_model(self, make: str = None) -> str:
        """
        Generate a random car model, optionally for a specific make.
        
        :param make: Optional car make to get model for
        :example: 'Camry'
        """
        if make and make in self.car_models:
            return self.random_element(self.car_models[make])
        elif make:
            return self.random_element(self.car_models["default"])
        else:
            # Pick a random make and then a model from it
            random_make = self.random_element(list(self.car_models.keys()))
            if random_make == "default":
                random_make = self.random_element([k for k in self.car_models.keys() if k != "default"])
            return self.random_element(self.car_models[random_make])

    def vehicle_make_model(self) -> str:
        """
        Generate a random vehicle make and model combination.
        
        :example: 'Toyota Camry'
        """
        make = self.car_make()
        model = self.car_model(make)
        return f"{make} {model}"

    def vehicle_year(self, min_year: int = 2000, max_year: int = 2024) -> int:
        """
        Generate a random vehicle year.
        
        :param min_year: Minimum year (default: 2000)
        :param max_year: Maximum year (default: 2024)
        :example: 2020
        """
        return self.random_int(min_year, max_year)

    def vehicle_color(self) -> str:
        """
        Generate a random vehicle color.
        
        :example: 'Silver'
        """
        return self.random_element(self.vehicle_colors)

    def fuel_type(self) -> str:
        """
        Generate a random fuel type.
        
        :example: 'Gasoline'
        """
        return self.random_element(self.fuel_types)

    def transmission(self) -> str:
        """
        Generate a random transmission type.
        
        :example: 'Automatic'
        """
        return self.random_element(self.transmission_types)

    def engine_size(self) -> str:
        """
        Generate a random engine size.
        
        :example: '2.5L'
        """
        return self.random_element(self.engine_sizes)

    def drive_type(self) -> str:
        """
        Generate a random drive type.
        
        :example: 'AWD'
        """
        return self.random_element(self.drive_types)

    def vehicle_feature(self) -> str:
        """
        Generate a random vehicle feature.
        
        :example: 'Sunroof'
        """
        return self.random_element(self.vehicle_features)

    def license_plate(self) -> str:
        """
        Generate a random license plate number.
        
        :example: 'ABC-1234'
        """
        pattern = self.random_element(self.license_plate_patterns)
        result = ""
        
        for char in pattern:
            if char == "#":
                result += str(self.random_int(0, 9))
            elif char == "A" or char == "B" or char == "C":
                result += self.random_element("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
            else:
                result += char
        
        return result

    def vin(self) -> str:
        """
        Generate a random Vehicle Identification Number (VIN).
        
        :example: '1HGBH41JXMN109186'
        """
        result = ""
        
        for char in self.vin_pattern:
            if char == "#":
                result += str(self.random_int(0, 9))
            else:
                result += char
        
        return result

    def vehicle_condition(self) -> str:
        """
        Generate a random vehicle condition.
        
        :example: 'Excellent'
        """
        return self.random_element(self.vehicle_conditions)

    def mileage(self, min_miles: int = 0, max_miles: int = 200000) -> int:
        """
        Generate a random mileage.
        
        :param min_miles: Minimum mileage (default: 0)
        :param max_miles: Maximum mileage (default: 200000)
        :example: 45000
        """
        return self.random_int(min_miles, max_miles)

    def vehicle_price(self, min_price: int = 5000, max_price: int = 100000) -> str:
        """
        Generate a random vehicle price.
        
        :param min_price: Minimum price (default: 5000)
        :param max_price: Maximum price (default: 100000)
        :example: '$25,999'
        """
        price = self.random_int(min_price, max_price)
        return f"${price:,}"

    def dealership_type(self) -> str:
        """
        Generate a random dealership type.
        
        :example: 'New Car Dealership'
        """
        return self.random_element(self.dealership_types)

    def insurance_type(self) -> str:
        """
        Generate a random insurance type.
        
        :example: 'Full Coverage'
        """
        return self.random_element(self.insurance_types)

    def mpg(self, min_mpg: int = 15, max_mpg: int = 50) -> int:
        """
        Generate a random MPG (Miles Per Gallon).
        
        :param min_mpg: Minimum MPG (default: 15)
        :param max_mpg: Maximum MPG (default: 50)
        :example: 28
        """
        return self.random_int(min_mpg, max_mpg)

    def horsepower(self, min_hp: int = 100, max_hp: int = 700) -> int:
        """
        Generate a random horsepower.
        
        :param min_hp: Minimum horsepower (default: 100)
        :param max_hp: Maximum horsepower (default: 700)
        :example: 250
        """
        return self.random_int(min_hp, max_hp)

    def seating_capacity(self, min_seats: int = 2, max_seats: int = 8) -> int:
        """
        Generate a random seating capacity.
        
        :param min_seats: Minimum seats (default: 2)
        :param max_seats: Maximum seats (default: 8)
        :example: 5
        """
        return self.random_int(min_seats, max_seats)



from randum.providers.bank import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``ro_RO`` locale."""

    country_code = "RO"
    bban_format = "????################"
    swift_bank_codes = (
        "NBOR",
        "ABNA",
        "BUCU",
        "ARBL",
        "MIND",
        "BPOS",
        "CARP",
        "RNCB",
        "BROM",
        "BITR",
        "BRDE",
        "BRMA",
        "BTRL",
        "DAFB",
        "MIRB",
        "CECE",
        "CITI",
        "CRCO",
        "FNNB",
        "EGNA",
        "BSEA",
        "EXIM",
        "UGBI",
        "HVBL",
        "INGB",
        "BREL",
        "CRDZ",
        "BNRB",
        "PIRB",
        "PORL",
        "MIRO",
        "RZBL",
        "RZBR",
        "ROIN",
        "WBAN",
        "TRFD",
        "TREZ",
        "BACX",
        "VBBU",
        "DARO",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = ("{{last_name}} {{first_name_female}}",)

    formats_male = ("{{last_name}} {{first_name_male}}",)

    formats = formats_male + formats_female

    # Source: https://studentsoftheworld.info/penpals/stats_fr.php?Pays=ALG
    # Last checked: 2025-09-27
    first_names_male = (
        "Abdelatif",
        "Abdelkader",
        "Abderaouf",
        "Abderrahmane",
        "Adam",
        "Adel",
        "Ahmed",
        "Akram",
        "Aladin",
        "Ali",
        "Amine",
        "Amir",
        "Anis",
        "Arezki",
        "Aymen",
        "Ayoub",
        "Chabane",
        "Cherif",
        "Djamel",
        "Fares",
        "Farid",
        "Farouk",
        "Fatteh",
        "Ferhat",
        "Fodil",
        "Ghilas",
        "Hamid",
        "Hamza",
        "Hocine",
        "Houcine",
        "Ibrahim",
        "Ilyes",
        "Kada",
        "Khaled",
        "Khalil",
        "Lamine",
        "Lotfi",
        "Malik",
        "Massinissa",
        "Mehdi",
        "Mohamed",
        "Mohand",
        "Mohammed",
        "Mouloud",
        "Mounir",
        "Mourad",
        "Moussa",
        "Mustapha",
        "Nacer",
        "Nadir",
        "Nassim",
        "Nazim",
        "Omar",
        "Oussama",
        "Ouanes",
        "Rabah",
        "Rachid",
        "Ramzi",
        "Riad",
        "Rida",
        "Rochdi",
        "Saad",
        "Said",
        "Salah",
        "Salim",
        "Sami",
        "Samir",
        "Samy",
        "Sofiane",
        "Soufiane",
        "Taha",
        "Walid",
        "Wassim",
        "Yacine",
        "Yahia",
        "Yanis",
        "Yasser",
        "Youba",
        "Youcef",
        "Younes",
        "Zakaria",
    )

    # Source: https://studentsoftheworld.info/penpals/stats_fr.php?Pays=ALG
    # Last checked: 2025-09-27
    first_names_female = (
        "Amani",
        "Amel",
        "Amina",
        "Amira",
        "Anaïs",
        "Anissa",
        "Asma",
        "Aya",
        "Bouchra",
        "Célia",
        "Céline",
        "Chanez",
        "Chiraz",
        "Dalia",
        "Dounia",
        "Farah",
        "Fatima",
        "Fella",
        "Feriel",
        "Fouzia",
        "Hadjer",
        "Hana",
        "Hania",
        "Hayat",
        "Houda",
        "Ikram",
        "Imene",
        "Ines",
        "Jasmine",
        "Kahina",
        "Katia",
        "Khadidja",
        "Leila",
        "Lila",
        "Lilia",
        "Lina",
        "Lisa",
        "Lydia",
        "Lyna",
        "Lynda",
        "Malak",
        "Manel",
        "Maria",
        "Marwa",
        "Maya",
        "Mélissa",
        "Meriem",
        "Mina",
        "Mira",
        "Myriam",
        "Nadia",
        "Narimane",
        "Nawal",
        "Nedjma",
        "Nesrine",
        "Nihad",
        "Nour",
        "Racha",
        "Rania",
        "Rim",
        "Rose",
        "Rym",
        "Ryma",
        "Sabrina",
        "Safia",
        "Sahra",
        "Salima",
        "Salma",
        "Samira",
        "Sara",
        "Sarah",
        "Selma",
        "Serine",
        "Sofia",
        "Sonia",
        "Sophia",
        "Souad",
        "Soumia",
        "Thinhinane",
        "Wafae",
        "Yasmine",
        "Yasmina",
        "Yousra",
        "Zina",
        "Zineb",
    )

    first_names = first_names_male + first_names_female

    # Source: https://fr.geneawiki.com/wiki/Noms_de_famille_alg%C3%A9riens
    # Last checked: 2025-09-27
    last_names = (
        "Abada",
        "Abbad",
        "Abbas",
        "Abbassi",
        "Abbes",
        "Abdi",
        "Abdelli",
        "Abdellaoui",
        "Abdelaziz",
        "Abdou",
        "Abed",
        "Abid",
        "Abou",
        "Abouda",
        "Aboukir",
        "Achour",
        "Achouri",
        "Adda",
        "Aissaoui",
        "Aissani",
        "Allal",
        "Allali",
        "Amara",
        "Amari",
        "Ameur",
        "Ammari",
        "Amrane",
        "Amrani",
        "Amrouche",
        "Amri",
        "Arab",
        "Aribi",
        "Attia",
        "Ayad",
        "Ayadi",
        "Azzouz",
        "Azizi",
        "Bacha",
        "Bahloul",
        "Bahri",
        "Bakhti",
        "Bakhouche",
        "Baghdadi",
        "Belarbi",
        "Belaid",
        "Belaidi",
        "Belbachir",
        "Belhadj",
        "Belkacem",
        "Belkacemi",
        "Belkadi",
        "Belkheir",
        "Belkhiri",
        "Benabdellah",
        "Benahmed",
        "Benali",
        "Benamar",
        "Benamara",
        "Benameur",
        "Benaicha",
        "Benaissa",
        "Benaouda",
        "Bencheikh",
        "Bensalem",
        "Bensaid",
        "Bensalah",
        "Benslimane",
        "Benyahia",
        "Benyamina",
        "Benmoussa",
        "Benyoucef",
        "Benziane",
        "Berkane",
        "Berkani",
        "Bettahar",
        "Bey",
        "Boubekeur",
        "Bouabdellah",
        "Bouafia",
        "Boualem",
        "Bouali",
        "Bouaziz",
        "Bouchama",
        "Bouchareb",
        "Boucetta",
        "Boudiaf",
        "Boudjemaa",
        "Boudraa",
        "Bouguerra",
        "Boukhari",
        "Boukhalfa",
        "Boukhatem",
        "Boumaza",
        "Boumediene",
        "Bounab",
        "Boussaid",
        "Boutaleb",
        "Bouziane",
        "Bouzid",
        "Bouzidi",
        "Brahimi",
        "Brahmi",
        "Chaib",
        "Chabane",
        "Charef",
        "Chaoui",
        "Chibani",
        "Chikh",
        "Chergui",
        "Cherif",
        "Cherifi",
        "Cheriet",
        "Cheikh",
        "Chellaoua",
        "Daoud",
        "Daoudi",
        "Dahmane",
        "Dahmani",
        "Derbal",
        "Derradji",
        "Derkaoui",
        "Derouiche",
        "Dib",
        "Diaf",
        "Dif",
        "Djebbar",
        "Djellal",
        "Djellouli",
        "Djoudi",
        "Fares",
        "Fellah",
        "Ferhat",
        "Filali",
        "Gacem",
        "Gasmi",
        "Ghazi",
        "Gharbi",
        "Gherbi",
        "Guessoum",
        "Guendouz",
        "Guerfi",
        "Hadjadj",
        "Hadji",
        "Haddad",
        "Haddouche",
        "Hachemi",
        "Hamel",
        "Hamadouche",
        "Hamadi",
        "Hamdani",
        "Hamdi",
        "Hamidi",
        "Hamlaoui",
        "Hammadi",
        "Hamoudi",
        "Hamza",
        "Hamzaoui",
        "Hassani",
        "Henni",
        "Hocine",
        "Houari",
        "Ikhlef",
        "Kaci",
        "Kaddour",
        "Kaddouri",
        "Kadi",
        "Kadri",
        "Kamel",
        "Kara",
        "Kebaili",
        "Kebir",
        "Khaldi",
        "Khaled",
        "Khelif",
        "Khelifa",
        "Khelifi",
        "Khelil",
        "Korichi",
        "Kouidri",
        "Laib",
        "Lakehal",
        "Lakhal",
        "Lakhdari",
        "Lamri",
        "Laouar",
        "Larbi",
        "Laribi",
        "Latreche",
        "Lahmar",
        "Lamri",
        "Laribi",
        "Latreche",
        "Lounis",
        "Loucif",
        "Madani",
        "Madi",
        "Mahmoudi",
        "Mahi",
        "Malki",
        "Malek",
        "Mansour",
        "Mansouri",
        "Maouche",
        "Makhlouf",
        "Makhloufi",
        "Mazouz",
        "Mazouzi",
        "Mebarki",
        "Mecheri",
        "Meftah",
        "Medjahed",
        "Meddah",
        "Meziane",
        "Meziani",
        "Mesbah",
        "Messaoudi",
        "Merabet",
        "Merah",
        "Merzougui",
        "Merzoug",
        "Mihoubi",
        "Miloudi",
        "Mimouni",
        "Mokadem",
        "Mokrane",
        "Mokrani",
        "Mokhtari",
        "Mohammedi",
        "Mostefaoui",
        "Morsli",
        "Moulay",
        "Moussa",
        "Moussaoui",
        "Nacer",
        "Naili",
        "Nasri",
        "Nedjar",
        "Nouar",
        "Noui",
        "Nouri",
        "Ouali",
        "Ouchene",
        "Otmani",
        "Rabhi",
        "Rachedi",
        "Rabia",
        "Rahmani",
        "Rahal",
        "Rahmouni",
        "Rahmoune",
        "Ramdani",
        "Rais",
        "Rezig",
        "Sabri",
        "Saci",
        "Saad",
        "Saadi",
        "Saidi",
        "Said",
        "Saidani",
        "Sahli",
        "Sahraoui",
        "Salem",
        "Salhi",
        "Salmi",
        "Salah",
        "Saoudi",
        "Sayah",
        "Seddiki",
        "Selami",
        "Senouci",
        "Slimani",
        "Smail",
        "Soudani",
        "Soltani",
        "Taibi",
        "Tabet",
        "Tahraoui",
        "Tahri",
        "Talbi",
        "Taleb",
        "Touati",
        "Touil",
        "Toumi",
        "Yahi",
        "Yahia",
        "Yahiaoui",
        "Yousfi",
        "Zaidi",
        "Zaoui",
        "Zeroual",
        "Zerrouki",
        "Ziane",
        "Ziani",
        "Zidane",
        "Zitouni",
        "Zouaoui",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = ("{{first_name_male}} {{last_name}}",)
    formats_female = ("{{first_name_female}} {{last_name}}",)
    formats = ("{{first_name}} {{last_name}}",)

    # https://www.in.pampers.com/pregnancy/baby-names/article/indian-baby-boys-names
    first_names_male = (
        "Aadi",
        "Aarav",
        "Aarnav",
        "Aarush",
        "Aayush",
        "Abdul",
        "Abeer",
        "Abhimanyu",
        "Abhiram",
        "Aditya",
        "Advaith",
        "Advay",
        "Advik",
        "Agastya",
        "Akshay",
        "Alexander",
        "Amol",
        "Anay",
        "Andrew",
        "Anirudh",
        "Anmol",
        "Ansh",
        "Anthony",
        "Arin",
        "Arjun",
        "Aryan",
        "Atharv",
        "Avi",
        "Ayaan",
        "Ayush",
        "Ayushman",
        "Azaan",
        "Azad",
        "Bachittar",
        "Bahadurjit",
        "Bakhshi",
        "Balendra",
        "Balhaar",
        "Baljiwan",
        "Balvan",
        "Balveer",
        "Banjeet",
        "Benjamin",
        "Brijesh",
        "Caleb",
        "Chaitanya",
        "Chakradev",
        "Chakradhar",
        "Champak",
        "Chanakya",
        "Chandran",
        "Chandresh",
        "Charan",
        "Charles",
        "Chatresh",
        "Chatura",
        "Christopher",
        "Daksh",
        "Dakshesh",
        "Dalbir",
        "Daniel",
        "Darpan",
        "Darsh",
        "David",
        "Dev",
        "Devansh",
        "Dhruv",
        "Dominic",
        "Ekalinga",
        "Ekansh",
        "Ekapad",
        "Ekaraj",
        "Ekavir",
        "Ekbal",
        "Elijah",
        "Ethan",
        "Falan",
        "Faqid",
        "Faraj",
        "Faras",
        "Farhan",
        "Fariq",
        "Faris",
        "Finn",
        "Fitan",
        "Fiyaz",
        "Frado",
        "Frederick",
        "Gabriel",
        "Gagan",
        "Gaurang",
        "Gaurav",
        "Gautam",
        "Gavin",
        "George",
        "Girik",
        "Girindra",
        "Girish",
        "Gopal",
        "Gunbir",
        "Guneet",
        "Hardik",
        "Harish",
        "Harrison",
        "Harsh",
        "Harshil",
        "Hemang",
        "Henry",
        "Hitesh",
        "Hredhaan",
        "Hritik",
        "Ikbal",
        "Imaran",
        "Indrajit",
        "Isaac",
        "Isaiah",
        "Ishaan",
        "Ishwar",
        "Jack",
        "Jackson",
        "Jacob",
        "Jagat",
        "Jagdish",
        "Jai",
        "Jairaj",
        "Jason",
        "Jatin",
        "Jeet",
        "Jeremiah",
        "Jonathan",
        "Joshua",
        "Kabir",
        "Kai",
        "Kalpit",
        "Karan",
        "Kevin",
        "Kiaan",
        "Krish",
        "Krishna",
        "Laban",
        "Laksh",
        "Lakshit",
        "Liam",
        "Logan",
        "Lohit",
        "Lucky",
        "Luke",
        "Maanas",
        "Maanav",
        "Madhav",
        "Manan",
        "Manbir",
        "Manthan",
        "Mason",
        "Matthew",
        "Max",
        "Michael",
        "Mitesh",
        "Mohammed",
        "Nachiket",
        "Naksh",
        "Nakul",
        "Nathan",
        "Nathaniel",
        "Naveen",
        "Neel",
        "Nicholas",
        "Nihal",
        "Nitesh",
        "Noah",
        "Ojas",
        "Oliver",
        "Om",
        "Omkaar",
        "Onkar",
        "Onveer",
        "Orinder",
        "Oscar",
        "Owen",
        "Parth",
        "Patrick",
        "Peter",
        "Pranav",
        "Praneel",
        "Pranit",
        "Pratyush",
        "Qabil",
        "Qadim",
        "Qarin",
        "Qasim",
        "Quincy",
        "Rachit",
        "Raghav",
        "Ranbir",
        "Ranveer",
        "Rayaan",
        "Rehaan",
        "Reyansh",
        "Rishi",
        "Robert",
        "Rohan",
        "Ronith",
        "Rudra",
        "Rushil",
        "Ryan",
        "Sai",
        "Saksham",
        "Samaksh",
        "Samar",
        "Samarth",
        "Samesh",
        "Samuel",
        "Sarthak",
        "Sathvik",
        "Shaurya",
        "Shivansh",
        "Siddharth",
        "Simon",
        "Tanay",
        "Tanish",
        "Tanveer",
        "Tarak",
        "Teerth",
        "Tejas",
        "Theodore",
        "Thomas",
        "Timothy",
        "Tristan",
        "Udant",
        "Udarsh",
        "Umang",
        "Upkaar",
        "Utkarsh",
        "Vedant",
        "Veer",
        "Victor",
        "Vihaan",
        "Vincent",
        "Viraj",
        "Vivaan",
        "Wahab",
        "Warinder",
        "Warjas",
        "Wazir",
        "William",
        "Wriddhish",
        "Wridesh",
        "Wyatt",
        "Xavier",
        "Yagnesh",
        "Yash",
        "Yatan",
        "Yatin",
        "Yug",
        "Yuvraj",
        "Zaid",
        "Zashil",
        "Zayan",
        "Zayyan",
        "Zehaan",
    )

    # https://www.pampers.com/en-us/pregnancy/baby-names/article/indian-girl-names
    first_names_female = (
        "Aachal",
        "Aadhya",
        "Aahana",
        "Aarini",
        "Aarna",
        "Aashi",
        "Abha",
        "Advika",
        "Adweta",
        "Adya",
        "Aishani",
        "Alka",
        "Amaira",
        "Amara",
        "Amrita",
        "Amruta",
        "Anamika",
        "Anika",
        "Anita",
        "Anjali",
        "Anusha",
        "Anvi",
        "Anya",
        "Aradhana",
        "Arunima",
        "Arya",
        "Ati",
        "Avni",
        "Baghyawati",
        "Barkha",
        "Bhanumati",
        "Bhavani",
        "Bhavika",
        "Bhavini",
        "Bhavna",
        "Bhavya",
        "Bimala",
        "Bina",
        "Bishakha",
        "Brinda",
        "Chaaya",
        "Chaitaly",
        "Chakrika",
        "Chaman",
        "Chameli",
        "Chanchal",
        "Chandani",
        "Charita",
        "Charvi",
        "Chasmum",
        "Chavvi",
        "Daksha",
        "Dalaja",
        "Damini",
        "Damyanti",
        "Darika",
        "Dayamai",
        "Dayita",
        "Deepa",
        "Devika",
        "Dhriti",
        "Dipta",
        "Divya",
        "Diya",
        "Edhitha",
        "Eesha",
        "Eiravati",
        "Ekaja",
        "Ekani",
        "Ekanta",
        "Ekantika",
        "Ekiya",
        "Ekta",
        "Eshana",
        "Eta",
        "Falak",
        "Falguni",
        "Forum",
        "Ganga",
        "Garima",
        "Gaurangi",
        "Gauri",
        "Gaurika",
        "Gautami",
        "Gayathri",
        "Geetika",
        "Hamsini",
        "Harinakshi",
        "Harini",
        "Harita",
        "Hema",
        "Hemal",
        "Hemangini",
        "Hemani",
        "Hiral",
        "Idika",
        "Ijaya",
        "Ikshita",
        "Inaya",
        "Indali",
        "Indira",
        "Ira",
        "Irya",
        "Isha",
        "Ishani",
        "Ishanvi",
        "Ishita",
        "Jagrati",
        "Jagvi",
        "Jalsa",
        "Janaki",
        "Janani",
        "Januja",
        "Janya",
        "Jasmit",
        "Jeevika",
        "Jhalak",
        "Jyoti",
        "Kala",
        "Kamala",
        "Kamya",
        "Kashish",
        "Kashvi",
        "Kavya",
        "Keya",
        "Krisha",
        "Krishna",
        "Kritika",
        "Ladli",
        "Lajita",
        "Lakshmi",
        "Lavanya",
        "Leela",
        "Leena",
        "Lekha",
        "Libni",
        "Lila",
        "Lipika",
        "Lopa",
        "Madhavi",
        "Mahika",
        "Manya",
        "Maya",
        "Meera",
        "Megha",
        "Meghana",
        "Mekhala",
        "Mitali",
        "Mohini",
        "Mugdha",
        "Nandini",
        "Neelima",
        "Neha",
        "Netra",
        "Nidhi",
        "Nidra",
        "Niharika",
        "Nikita",
        "Nilima",
        "Nimrat",
        "Nirja",
        "Nisha",
        "Nitara",
        "Odika",
        "Oeshi",
        "Ojasvi",
        "Omaja",
        "Omisha",
        "Omya",
        "Oni",
        "Osha",
        "Oviya",
        "Pahal",
        "Pallavi",
        "Panini",
        "Pavani",
        "Pooja",
        "Prisha",
        "Priya",
        "Pushti",
        "Qushi",
        "Raagini",
        "Rachana",
        "Rachita",
        "Radha",
        "Radhika",
        "Rajata",
        "Rajeshri",
        "Raksha",
        "Reva",
        "Ria",
        "Ridhi",
        "Riya",
        "Saanvi",
        "Sachi",
        "Sai",
        "Sanaya",
        "Sanya",
        "Sara",
        "Saumya",
        "Shivani",
        "Shravya",
        "Siya",
        "Sneha",
        "Sudiksha",
        "Suhani",
        "Tamanna",
        "Tanmayi",
        "Tanvi",
        "Tara",
        "Tripti",
        "Triveni",
        "Triya",
        "Turvi",
        "Ubika",
        "Ucchal",
        "Udyati",
        "Unnati",
        "Unni",
        "Upadhriti",
        "Upasna",
        "Upma",
        "Urishilla",
        "Urmi",
        "Urvashi",
        "Urvi",
        "Vaishnavi",
        "Vamakshi",
        "Vansha",
        "Vanya",
        "Varenya",
        "Varsha",
        "Vasana",
        "Vasatika",
        "Vasudha",
        "Veda",
        "Vedhika",
        "Vedika",
        "Vidhi",
        "Vinaya",
        "Vrinda",
        "Vrishti",
        "Vritti",
        "Vyanjana",
        "Waida",
        "Wakeeta",
        "Warda",
        "Warhi",
        "Watika",
        "Widisha",
        "Wishi",
        "Xalak",
        "Xiti",
        "Yachana",
        "Yadavi",
        "Yahvi",
        "Yamini",
        "Yashasvi",
        "Yashawini",
        "Yashica",
        "Yashoda",
        "Yashodhara",
        "Yashvi",
        "Yasti",
        "Yauvani",
        "Yochana",
        "Yoshita",
        "Yutika",
        "Zaitra",
        "Zansi",
        "Zarna",
        "Zilmil",
        "Zinal",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Acharya",
        "Agarwal",
        "Agate",
        "Aggarwal",
        "Agrawal",
        "Ahluwalia",
        "Ahuja",
        "Amble",
        "Anand",
        "Andra",
        "Anne",
        "Apte",
        "Arora",
        "Arya",
        "Atwal",
        "Aurora",
        "Babu",
        "Badal",
        "Badami",
        "Bahl",
        "Bahri",
        "Bail",
        "Bains",
        "Bajaj",
        "Bajwa",
        "Bakshi",
        "Bal",
        "Bala",
        "Bala",
        "Balakrishnan",
        "Balan",
        "Balasubramanian",
        "Balay",
        "Bali",
        "Bandi",
        "Banerjee",
        "Banik",
        "Bansal",
        "Barad",
        "Barad",
        "Baral",
        "Baria",
        "Barman",
        "Basak",
        "Bassi",
        "Basu",
        "Bath",
        "Batra",
        "Batta",
        "Bava",
        "Bawa",
        "Bedi",
        "Behl",
        "Ben",
        "Bera",
        "Bhagat",
        "Bhakta",
        "Bhalla",
        "Bhandari",
        "Bhardwaj",
        "Bhargava",
        "Bhasin",
        "Bhat",
        "Bhatia",
        "Bhatnagar",
        "Bhatt",
        "Bhattacharyya",
        "Bhatti",
        "Bhavsar",
        "Bir",
        "Biswas",
        "Boase",
        "Bobal",
        "Bora",
        "Bora",
        "Borah",
        "Borde",
        "Borra",
        "Bose",
        "Brahmbhatt",
        "Brar",
        "Buch",
        "Buch",
        "Bumb",
        "Butala",
        "Chacko",
        "Chad",
        "Chada",
        "Chadha",
        "Chahal",
        "Chakrabarti",
        "Chakraborty",
        "Chana",
        "Chand",
        "Chanda",
        "Chander",
        "Chandra",
        "Chandran",
        "Char",
        "Chatterjee",
        "Chaudhari",
        "Chaudhary",
        "Chaudhry",
        "Chaudhuri",
        "Chaudry",
        "Chauhan",
        "Chawla",
        "Cheema",
        "Cherian",
        "Chhabra",
        "Chokshi",
        "Chopra",
        "Choudhary",
        "Choudhry",
        "Choudhury",
        "Chowdhury",
        "Comar",
        "Contractor",
        "D’Alia",
        "Dada",
        "Dalal",
        "Dani",
        "Dar",
        "Dara",
        "Dara",
        "Das",
        "Dasgupta",
        "Dash",
        "Dass",
        "Date",
        "Datta",
        "Dave",
        "Dayal",
        "De",
        "Deep",
        "Deo",
        "Deol",
        "Desai",
        "Deshmukh",
        "Deshpande",
        "Devan",
        "Devi",
        "Dewan",
        "Dey",
        "Dhaliwal",
        "Dhar",
        "Dhar",
        "Dhawan",
        "Dhillon",
        "Dhingra",
        "Din",
        "Divan",
        "Dixit",
        "Doctor",
        "Dora",
        "Doshi",
        "Dua",
        "Dube",
        "Dubey",
        "Dugal",
        "Dugar",
        "Dugar",
        "Dutt",
        "Dutta",
        "Dyal",
        "Edwin",
        "Gaba",
        "Gade",
        "Gala",
        "Gandhi",
        "Ganesan",
        "Ganesh",
        "Ganguly",
        "Gara",
        "Garde",
        "Garg",
        "Gera",
        "Ghose",
        "Ghosh",
        "Gill",
        "Goda",
        "Goel",
        "Gokhale",
        "Gola",
        "Gole",
        "Golla",
        "Gopal",
        "Goswami",
        "Gour",
        "Goyal",
        "Grewal",
        "Grover",
        "Guha",
        "Gulati",
        "Gupta",
        "Halder",
        "Handa",
        "Hans",
        "Hari",
        "Hayer",
        "Hayre",
        "Hegde",
        "Hora",
        "Issac",
        "Iyengar",
        "Iyer",
        "Jaggi",
        "Jain",
        "Jani",
        "Jayaraman",
        "Jha",
        "Jhaveri",
        "Johal",
        "Joshi",
        "Kadakia",
        "Kade",
        "Kakar",
        "Kala",
        "Kala",
        "Kala",
        "Kale",
        "Kalita",
        "Kalla",
        "Kamdar",
        "Kanda",
        "Kannan",
        "Kant",
        "Kapadia",
        "Kapoor",
        "Kapur",
        "Kar",
        "Kara",
        "Karan",
        "Kari",
        "Karnik",
        "Karpe",
        "Kashyap",
        "Kata",
        "Kaul",
        "Kaur",
        "Keer",
        "Keer",
        "Khalsa",
        "Khanna",
        "Khare",
        "Khatri",
        "Khosla",
        "Khurana",
        "Kibe",
        "Kohli",
        "Konda",
        "Korpal",
        "Koshy",
        "Kota",
        "Kothari",
        "Krish",
        "Krishna",
        "Krishnamurthy",
        "Krishnan",
        "Kulkarni",
        "Kumar",
        "Kumer",
        "Kunda",
        "Kurian",
        "Kuruvilla",
        "Lad",
        "Lad",
        "Lal",
        "Lala",
        "Lall",
        "Lalla",
        "Lanka",
        "Lata",
        "Loke",
        "Loyal",
        "Luthra",
        "Madan",
        "Madan",
        "Magar",
        "Mahajan",
        "Mahal",
        "Maharaj",
        "Majumdar",
        "Malhotra",
        "Mall",
        "Mallick",
        "Mammen",
        "Mand",
        "Manda",
        "Mandal",
        "Mander",
        "Mane",
        "Mangal",
        "Mangat",
        "Mani",
        "Mani",
        "Mann",
        "Mannan",
        "Manne",
        "Master",
        "Memon",
        "Menon",
        "Merchant",
        "Minhas",
        "Mishra",
        "Misra",
        "Mistry",
        "Mital",
        "Mitra",
        "Mittal",
        "Mitter",
        "Modi",
        "Mody",
        "Mohan",
        "Mohanty",
        "Morar",
        "More",
        "Mukherjee",
        "Mukhopadhyay",
        "Muni",
        "Munshi",
        "Murthy",
        "Murty",
        "Mutti",
        "Nadig",
        "Nadkarni",
        "Nagar",
        "Nagarajan",
        "Nagi",
        "Nagy",
        "Naidu",
        "Naik",
        "Nair",
        "Nanda",
        "Narain",
        "Narang",
        "Narasimhan",
        "Narayan",
        "Narayanan",
        "Narula",
        "Natarajan",
        "Nath",
        "Natt",
        "Nayak",
        "Nayar",
        "Nazareth",
        "Nigam",
        "Nori",
        "Oak",
        "Om",
        "Oommen",
        "Oza",
        "Padmanabhan",
        "Pai",
        "Pal",
        "Palan",
        "Pall",
        "Palla",
        "Palla",
        "Panchal",
        "Pandey",
        "Pandit",
        "Pandya",
        "Pant",
        "Parekh",
        "Parikh",
        "Parmar",
        "Parmer",
        "Parsa",
        "Patel",
        "Pathak",
        "Patil",
        "Patla",
        "Patla",
        "Pau",
        "Peri",
        "Pillai",
        "Pillay",
        "Pingle",
        "Prabhakar",
        "Prabhu",
        "Pradhan",
        "Prakash",
        "Prasad",
        "Prashad",
        "Puri",
        "Purohit",
        "Radhakrishnan",
        "Raghavan",
        "Rai",
        "Raj",
        "Raja",
        "Rajagopal",
        "Rajagopalan",
        "Rajan",
        "Raju",
        "Ram",
        "Rama",
        "Ramachandran",
        "Ramakrishnan",
        "Raman",
        "Ramanathan",
        "Ramaswamy",
        "Ramesh",
        "Rana",
        "Randhawa",
        "Ranganathan",
        "Rao",
        "Rastogi",
        "Ratta",
        "Rattan",
        "Ratti",
        "Rau",
        "Raval",
        "Ravel",
        "Ravi",
        "Ray",
        "Reddy",
        "Rege",
        "Rout",
        "Roy",
        "Sabharwal",
        "Sachar",
        "Sachdev",
        "Sachdeva",
        "Sagar",
        "Saha",
        "Sahni",
        "Sahota",
        "Saini",
        "Salvi",
        "Sama",
        "Sami",
        "Sampath",
        "Samra",
        "Sandal",
        "Sandhu",
        "Sane",
        "Sangha",
        "Sanghvi",
        "Sani",
        "Sankar",
        "Sankaran",
        "Sant",
        "Saraf",
        "Saran",
        "Sarin",
        "Sarkar",
        "Sarma",
        "Sarna",
        "Sarraf",
        "Sastry",
        "Sathe",
        "Savant",
        "Sawhney",
        "Saxena",
        "Sehgal",
        "Sekhon",
        "Sem",
        "Sen",
        "Sengupta",
        "Seshadri",
        "Seth",
        "Sethi",
        "Setty",
        "Sha",
        "Shah",
        "Shan",
        "Shankar",
        "Shanker",
        "Sharaf",
        "Sharma",
        "Shenoy",
        "Shere",
        "Sheth",
        "Shetty",
        "Shroff",
        "Shukla",
        "Sibal",
        "Sidhu",
        "Singh",
        "Singhal",
        "Sinha",
        "Sodhi",
        "Solanki",
        "Som",
        "Soman",
        "Soni",
        "Sood",
        "Sridhar",
        "Srinivas",
        "Srinivasan",
        "Srivastava",
        "Subramaniam",
        "Subramanian",
        "Sule",
        "Sundaram",
        "Sunder",
        "Sur",
        "Sura",
        "Suresh",
        "Suri",
        "Swaminathan",
        "Swamy",
        "Tailor",
        "Tak",
        "Talwar",
        "Tandon",
        "Taneja",
        "Tank",
        "Tara",
        "Tata",
        "Tella",
        "Thaker",
        "Thakkar",
        "Thakur",
        "Thaman",
        "Tiwari",
        "Toor",
        "Tripathi",
        "Trivedi",
        "Upadhyay",
        "Uppal",
        "Vaidya",
        "Vala",
        "Varghese",
        "Varkey",
        "Varma",
        "Varty",
        "Varughese",
        "Vasa",
        "Venkataraman",
        "Venkatesh",
        "Verma",
        "Vig",
        "Virk",
        "Viswanathan",
        "Vohra",
        "Vora",
        "Vyas",
        "Wable",
        "Wadhwa",
        "Wagle",
        "Wali",
        "Wali",
        "Walia",
        "Walla",
        "Warrior",
        "Wason",
        "Yadav",
        "Yogi",
        "Yohannan",
        "Zacharia",
        "Zachariah",
    )



from .. import BaseProvider

localized = False


class Provider(BaseProvider):
    """Provider for book and literature-related data."""

    # Literary genres
    literary_genres = [
        "Fiction", "Non-Fiction", "Mystery", "Thriller", "Romance",
        "Science Fiction", "Fantasy", "Horror", "Historical Fiction",
        "Literary Fiction", "Contemporary Fiction", "Young Adult",
        "Children's Literature", "Biography", "Autobiography", "Memoir",
        "Self-Help", "Business", "History", "Philosophy", "Psychology",
        "Science", "Travel", "Cooking", "Art", "Poetry", "Drama",
        "Crime", "Detective", "Adventure", "Western", "Dystopian",
        "Paranormal", "Urban Fantasy", "Epic Fantasy", "Space Opera",
        "Cyberpunk", "Steampunk", "Magical Realism", "Satire", "Humor"
    ]

    # Publishers
    publishers = [
        "Penguin Random House", "HarperCollins", "Simon & Schuster",
        "Hachette Book Group", "Macmillan Publishers", "Scholastic",
        "Pearson", "Wiley", "Oxford University Press", "Cambridge University Press",
        "Bloomsbury", "Vintage Books", "Knopf", "Crown Publishing",
        "Little, Brown and Company", "Doubleday", "Viking Press",
        "Farrar, Straus and Giroux", "Grove Press", "New Directions",
        "Tor Books", "Del Rey Books", "Ace Books", "DAW Books",
        "Orbit Books", "Baen Books", "Harlequin", "Kensington Publishing"
    ]

    # Book formats
    book_formats = [
        "Hardcover", "Paperback", "Mass Market Paperback", "eBook",
        "Audiobook", "Kindle Edition", "Large Print", "Board Book",
        "Leather Bound", "Box Set", "Collector's Edition"
    ]

    # Book series templates
    series_templates = [
        "The {{word}} Chronicles",
        "The {{word}} Saga",
        "The {{word}} Trilogy",
        "{{word}} Series",
        "The {{word}} Cycle",
        "Tales of {{word}}",
        "The {{word}} Collection",
        "{{word}} Adventures",
        "The {{word}} Legacy",
        "{{word}} and {{word_2}}",
    ]

    # Book title templates
    title_templates = [
        "The {{word}}",
        "{{word}} and {{word_2}}",
        "The {{word}} of {{word_2}}",
        "{{word}}: A {{genre}} Story",
        "The Last {{word}}",
        "The Secret {{word}}",
        "A {{word}} in {{word_2}}",
        "The {{word}}'s {{word_2}}",
        "Beyond {{word}}",
        "{{word}} Rising",
        "The {{word}} Affair",
        "{{word}} Dreams",
        "Shadows of {{word}}",
        "The {{word}} Prophecy",
        "{{word}} Awakening",
    ]

    # Title words
    title_words = [
        "Shadow", "Light", "Dark", "Night", "Dawn", "Dusk", "Moon", "Sun",
        "Star", "Fire", "Ice", "Storm", "Wind", "Rain", "Thunder", "Sky",
        "Ocean", "Mountain", "River", "Forest", "Garden", "Kingdom", "Empire",
        "Crown", "Throne", "Sword", "Shield", "Dragon", "Phoenix", "Wolf",
        "Lion", "Eagle", "Rose", "Thorn", "Blood", "Stone", "Crystal",
        "Silver", "Golden", "Crimson", "Emerald", "Sapphire", "Ruby",
        "Time", "Destiny", "Fate", "Dream", "Memory", "Secret", "Mystery",
        "Truth", "Lie", "Hope", "Fear", "Love", "Death", "Life", "Soul"
    ]

    # Chapter title templates
    chapter_templates = [
        "Chapter {{number}}: {{word}}",
        "{{number}}. The {{word}}",
        "Part {{number}}: {{word}} and {{word_2}}",
        "{{word}}",
        "The {{word}} Begins",
        "{{word}}'s End",
    ]

    # Literary awards
    literary_awards = [
        "Pulitzer Prize", "Nobel Prize in Literature", "Man Booker Prize",
        "National Book Award", "Hugo Award", "Nebula Award",
        "Edgar Award", "Newbery Medal", "Caldecott Medal",
        "PEN/Faulkner Award", "Costa Book Award", "Baileys Women's Prize",
        "National Book Critics Circle Award", "Los Angeles Times Book Prize",
        "Goodreads Choice Award", "New York Times Bestseller"
    ]

    # Reading levels
    reading_levels = [
        "Early Reader", "Chapter Book", "Middle Grade", "Young Adult",
        "New Adult", "Adult", "Advanced"
    ]

    # Book conditions
    book_conditions = [
        "New", "Like New", "Very Good", "Good", "Acceptable",
        "Fair", "Poor", "Collectible", "First Edition", "Signed Copy"
    ]

    # Literary terms
    literary_terms = [
        "Protagonist", "Antagonist", "Plot", "Setting", "Theme",
        "Conflict", "Resolution", "Climax", "Foreshadowing", "Flashback",
        "Metaphor", "Simile", "Allegory", "Symbolism", "Irony",
        "Point of View", "Narrator", "Character Development", "Dialogue",
        "Exposition", "Rising Action", "Falling Action", "Denouement"
    ]

    # Book clubs/reading groups
    book_club_types = [
        "Book Club", "Reading Group", "Literary Circle", "Book Discussion",
        "Reading Society", "Literature Club", "Book Lovers Group"
    ]

    # Bookstore types
    bookstore_types = [
        "Independent Bookstore", "Chain Bookstore", "Used Bookstore",
        "Antiquarian Bookstore", "Online Bookstore", "University Bookstore",
        "Specialty Bookstore", "Comic Book Store", "Children's Bookstore"
    ]

    def book_title(self) -> str:
        """
        Generate a random book title.
        
        :example: 'The Shadow of Time'
        """
        template = self.random_element(self.title_templates)
        result = template
        
        # Replace placeholders
        if "{{word}}" in result:
            result = result.replace("{{word}}", self.random_element(self.title_words))
        if "{{word_2}}" in result:
            result = result.replace("{{word_2}}", self.random_element(self.title_words))
        if "{{genre}}" in result:
            result = result.replace("{{genre}}", self.random_element(self.literary_genres))
        
        return result

    def author_name(self) -> str:
        """
        Generate a random author name.
        
        :example: 'Jane Smith'
        """
        return self.generator.parse("{{first_name}} {{last_name}}")

    def literary_genre(self) -> str:
        """
        Generate a random literary genre.
        
        :example: 'Science Fiction'
        """
        return self.random_element(self.literary_genres)

    def publisher(self) -> str:
        """
        Generate a random publisher name.
        
        :example: 'Penguin Random House'
        """
        return self.random_element(self.publishers)

    def book_format(self) -> str:
        """
        Generate a random book format.
        
        :example: 'Hardcover'
        """
        return self.random_element(self.book_formats)

    def book_series(self) -> str:
        """
        Generate a random book series name.
        
        :example: 'The Dragon Chronicles'
        """
        template = self.random_element(self.series_templates)
        result = template
        
        if "{{word}}" in result:
            result = result.replace("{{word}}", self.random_element(self.title_words))
        if "{{word_2}}" in result:
            result = result.replace("{{word_2}}", self.random_element(self.title_words))
        
        return result

    def chapter_title(self, chapter_number: int = None) -> str:
        """
        Generate a random chapter title.
        
        :param chapter_number: Optional chapter number (default: random 1-30)
        :example: 'Chapter 5: The Shadow'
        """
        if chapter_number is None:
            chapter_number = self.random_int(1, 30)
        
        template = self.random_element(self.chapter_templates)
        result = template
        
        if "{{number}}" in result:
            result = result.replace("{{number}}", str(chapter_number))
        if "{{word}}" in result:
            result = result.replace("{{word}}", self.random_element(self.title_words))
        if "{{word_2}}" in result:
            result = result.replace("{{word_2}}", self.random_element(self.title_words))
        
        return result

    def literary_award(self) -> str:
        """
        Generate a random literary award name.
        
        :example: 'Pulitzer Prize'
        """
        return self.random_element(self.literary_awards)

    def reading_level(self) -> str:
        """
        Generate a random reading level.
        
        :example: 'Young Adult'
        """
        return self.random_element(self.reading_levels)

    def book_condition(self) -> str:
        """
        Generate a random book condition.
        
        :example: 'Like New'
        """
        return self.random_element(self.book_conditions)

    def literary_term(self) -> str:
        """
        Generate a random literary term.
        
        :example: 'Protagonist'
        """
        return self.random_element(self.literary_terms)

    def book_club_type(self) -> str:
        """
        Generate a random book club type.
        
        :example: 'Book Club'
        """
        return self.random_element(self.book_club_types)

    def bookstore_type(self) -> str:
        """
        Generate a random bookstore type.
        
        :example: 'Independent Bookstore'
        """
        return self.random_element(self.bookstore_types)

    def publication_year(self, min_year: int = 1900, max_year: int = 2024) -> int:
        """
        Generate a random publication year.
        
        :param min_year: Minimum year (default: 1900)
        :param max_year: Maximum year (default: 2024)
        :example: 2015
        """
        return self.random_int(min_year, max_year)

    def page_count(self, min_pages: int = 100, max_pages: int = 800) -> int:
        """
        Generate a random page count.
        
        :param min_pages: Minimum pages (default: 100)
        :param max_pages: Maximum pages (default: 800)
        :example: 342
        """
        return self.random_int(min_pages, max_pages)

    def book_price(self, min_price: float = 5.0, max_price: float = 50.0) -> str:
        """
        Generate a random book price.
        
        :param min_price: Minimum price (default: 5.0)
        :param max_price: Maximum price (default: 50.0)
        :example: '$24.99'
        """
        price = self.random_int(int(min_price * 100), int(max_price * 100)) / 100
        return f"${price:.2f}"

    def isbn(self) -> str:
        """
        Generate a random ISBN-13 number.
        
        :example: '978-3-16-148410-0'
        """
        # Generate ISBN-13 format
        prefix = "978"
        group = str(self.random_int(0, 9))
        publisher_code = str(self.random_int(10, 99))
        title_code = str(self.random_int(10000, 99999))
        
        # Calculate check digit (simplified)
        isbn_without_check = f"{prefix}{group}{publisher_code}{title_code}"
        check_digit = self.random_int(0, 9)
        
        return f"{prefix}-{group}-{publisher_code}-{title_code}-{check_digit}"

    def book_rating(self, min_rating: float = 1.0, max_rating: float = 5.0) -> float:
        """
        Generate a random book rating.
        
        :param min_rating: Minimum rating (default: 1.0)
        :param max_rating: Maximum rating (default: 5.0)
        :example: 4.2
        """
        rating = self.random_int(int(min_rating * 10), int(max_rating * 10)) / 10
        return rating

    def review_count(self, min_reviews: int = 10, max_reviews: int = 10000) -> int:
        """
        Generate a random review count.
        
        :param min_reviews: Minimum reviews (default: 10)
        :param max_reviews: Maximum reviews (default: 10000)
        :example: 1523
        """
        return self.random_int(min_reviews, max_reviews)

    def edition(self) -> str:
        """
        Generate a random book edition.
        
        :example: '2nd Edition'
        """
        editions = [
            "1st Edition", "2nd Edition", "3rd Edition", "4th Edition",
            "5th Edition", "Revised Edition", "Special Edition",
            "Anniversary Edition", "Deluxe Edition", "Collector's Edition",
            "Limited Edition", "Illustrated Edition", "Annotated Edition"
        ]
        return self.random_element(editions)

    def language(self) -> str:
        """
        Generate a random book language.
        
        :example: 'English'
        """
        languages = [
            "English", "Spanish", "French", "German", "Italian", "Portuguese",
            "Russian", "Chinese", "Japanese", "Korean", "Arabic", "Hindi",
            "Dutch", "Swedish", "Norwegian", "Danish", "Polish", "Turkish"
        ]
        return self.random_element(languages)



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    # Source: https://en.wikipedia.org/wiki/Telephone_numbers_in_Jordan

    cellphone_formats = (
        "+9627{{operator_id}}#######",
        "+962 7 {{operator_id}}### ####",
        "07{{operator_id}}#######",
        "07{{operator_id}} ### ####",
    )

    telephone_formats = (
        "+962{{area_code}}#######",
        "+962 {{area_code}} ### ####",
        "0{{area_code}}#######",
        "0{{area_code}} ### ####",
    )

    services_phones_formats = (
        "9##",
        "12##",
        "13##",
        "14##",
    )

    formats = cellphone_formats + telephone_formats + services_phones_formats

    def operator_id(self) -> str:
        return self.random_element(
            [
                "4",
                "7",
                "8",
                "9",
            ]
        )

    def area_code(self) -> str:
        return self.random_element(
            [
                "2",
                "3",
                "5",
                "6",
                "7",
            ]
        )

    def cellphone_number(self) -> str:
        pattern: str = self.random_element(self.cellphone_formats)
        return self.numerify(self.generator.parse(pattern))

    def telephone_number(self) -> str:
        pattern: str = self.random_element(self.telephone_formats)
        return self.numerify(self.generator.parse(pattern))

    def service_phone_number(self) -> str:
        pattern: str = self.random_element(self.services_phones_formats)
        return self.numerify(self.generator.parse(pattern))

    def phone_number(self) -> str:
        pattern: str = self.random_element(self.formats)
        return self.numerify(self.generator.parse(pattern))



from importlib import import_module

from .utils.loading import find_available_locales, find_available_providers

DEFAULT_LOCALE = "en_US"

META_PROVIDERS_MODULES = [
    "randum.providers",
]

PROVIDERS = find_available_providers([import_module(path) for path in META_PROVIDERS_MODULES])

AVAILABLE_LOCALES = find_available_locales(PROVIDERS)



"""Education provider for Randum."""

from .. import BaseProvider


class Provider(BaseProvider):
    """Education provider for generating fake education-related data."""

    # Degree types
    degree_types = (
        "Associate",
        "Bachelor",
        "Master",
        "Doctorate",
        "Certificate",
        "Diploma",
    )

    # Degree abbreviations
    degree_abbrs = (
        "AA", "AS", "AAS",  # Associate degrees
        "BA", "BS", "BFA", "BBA", "BSc", "BEng",  # Bachelor degrees
        "MA", "MS", "MBA", "MFA", "MSc", "MEng", "MPhil",  # Master degrees
        "PhD", "EdD", "DBA", "MD", "JD", "DDS",  # Doctorate degrees
    )

    # Fields of study
    fields_of_study = (
        "Computer Science",
        "Business Administration",
        "Engineering",
        "Psychology",
        "Biology",
        "Chemistry",
        "Physics",
        "Mathematics",
        "English Literature",
        "History",
        "Political Science",
        "Economics",
        "Sociology",
        "Anthropology",
        "Philosophy",
        "Art",
        "Music",
        "Theater",
        "Education",
        "Nursing",
        "Medicine",
        "Law",
        "Architecture",
        "Environmental Science",
        "Communications",
        "Marketing",
        "Finance",
        "Accounting",
        "Information Technology",
        "Data Science",
        "Mechanical Engineering",
        "Electrical Engineering",
        "Civil Engineering",
        "Chemical Engineering",
        "Biomedical Engineering",
        "Aerospace Engineering",
        "Graphic Design",
        "Fashion Design",
        "Interior Design",
        "Journalism",
        "Public Relations",
        "International Relations",
        "Criminal Justice",
        "Social Work",
        "Public Health",
        "Nutrition",
        "Kinesiology",
        "Sports Management",
        "Hospitality Management",
        "Culinary Arts",
    )

    # University names
    university_names = (
        "State University",
        "Tech Institute",
        "College of Arts and Sciences",
        "University of Technology",
        "Metropolitan University",
        "National University",
        "International University",
        "Polytechnic Institute",
        "Academy of Sciences",
        "Institute of Technology",
        "Community College",
        "Liberal Arts College",
        "Business School",
        "Medical School",
        "Law School",
        "Engineering College",
        "Agricultural University",
        "Maritime Academy",
        "Aviation Institute",
        "Culinary Institute",
    )

    # School levels
    school_levels = (
        "Elementary School",
        "Middle School",
        "High School",
        "College",
        "University",
        "Graduate School",
        "Vocational School",
        "Trade School",
        "Community College",
        "Online University",
    )

    # Academic subjects
    academic_subjects = (
        "Mathematics",
        "English",
        "Science",
        "History",
        "Geography",
        "Physics",
        "Chemistry",
        "Biology",
        "Literature",
        "Writing",
        "Reading",
        "Social Studies",
        "Physical Education",
        "Art",
        "Music",
        "Drama",
        "Foreign Language",
        "Computer Science",
        "Economics",
        "Government",
        "Psychology",
        "Sociology",
        "Philosophy",
        "Statistics",
        "Calculus",
        "Algebra",
        "Geometry",
        "Trigonometry",
        "Environmental Science",
        "Health",
    )

    # Course names
    course_prefixes = (
        "Introduction to",
        "Advanced",
        "Fundamentals of",
        "Principles of",
        "Theory of",
        "Applied",
        "Modern",
        "Contemporary",
        "Classical",
        "Comparative",
        "Experimental",
        "Research Methods in",
        "History of",
        "Topics in",
        "Seminar in",
    )

    # Academic titles
    academic_titles = (
        "Professor",
        "Associate Professor",
        "Assistant Professor",
        "Lecturer",
        "Instructor",
        "Teaching Assistant",
        "Research Assistant",
        "Adjunct Professor",
        "Visiting Professor",
        "Professor Emeritus",
        "Dean",
        "Department Chair",
        "Academic Advisor",
        "Tutor",
        "Graduate Student",
    )

    # Extracurricular activities
    extracurricular_activities = (
        "Student Government",
        "Debate Club",
        "Drama Club",
        "Chess Club",
        "Robotics Club",
        "Science Club",
        "Math Club",
        "Art Club",
        "Music Band",
        "Orchestra",
        "Choir",
        "Dance Team",
        "Yearbook Committee",
        "School Newspaper",
        "Literary Magazine",
        "Environmental Club",
        "Volunteer Club",
        "Honor Society",
        "Language Club",
        "Photography Club",
        "Film Club",
        "Coding Club",
        "Gaming Club",
        "Book Club",
        "Model UN",
    )

    # Academic honors
    academic_honors = (
        "Dean's List",
        "Honor Roll",
        "Summa Cum Laude",
        "Magna Cum Laude",
        "Cum Laude",
        "Valedictorian",
        "Salutatorian",
        "National Merit Scholar",
        "Presidential Scholar",
        "Academic Excellence Award",
        "Outstanding Student Award",
        "Research Excellence Award",
        "Teaching Excellence Award",
        "Leadership Award",
        "Service Award",
    )

    # Grading systems
    letter_grades = ("A+", "A", "A-", "B+", "B", "B-", "C+", "C", "C-", "D+", "D", "D-", "F")
    
    gpa_scales = ("4.0", "5.0", "100")

    # Academic terms
    academic_terms = (
        "Fall Semester",
        "Spring Semester",
        "Summer Session",
        "Winter Term",
        "Quarter 1",
        "Quarter 2",
        "Quarter 3",
        "Quarter 4",
        "Trimester 1",
        "Trimester 2",
        "Trimester 3",
    )

    # Campus facilities
    campus_facilities = (
        "Library",
        "Student Center",
        "Recreation Center",
        "Dining Hall",
        "Dormitory",
        "Laboratory",
        "Auditorium",
        "Stadium",
        "Gymnasium",
        "Computer Lab",
        "Art Studio",
        "Music Hall",
        "Theater",
        "Bookstore",
        "Health Center",
        "Career Center",
        "Counseling Center",
        "Research Center",
        "Conference Center",
        "Alumni Center",
    )

    # Student classifications
    student_classifications = (
        "Freshman",
        "Sophomore",
        "Junior",
        "Senior",
        "Graduate Student",
        "Undergraduate",
        "Postgraduate",
        "Doctoral Candidate",
        "Transfer Student",
        "International Student",
        "Part-time Student",
        "Full-time Student",
        "Online Student",
        "Distance Learning Student",
    )

    # Academic departments
    academic_departments = (
        "Department of Computer Science",
        "Department of Mathematics",
        "Department of Physics",
        "Department of Chemistry",
        "Department of Biology",
        "Department of English",
        "Department of History",
        "Department of Psychology",
        "Department of Sociology",
        "Department of Economics",
        "Department of Political Science",
        "Department of Philosophy",
        "School of Business",
        "School of Engineering",
        "School of Medicine",
        "School of Law",
        "School of Education",
        "School of Arts",
        "College of Liberal Arts",
        "College of Sciences",
    )

    def degree_type(self) -> str:
        """
        Generate a random degree type.
        
        :example: 'Bachelor'
        """
        return self.random_element(self.degree_types)

    def degree_abbr(self) -> str:
        """
        Generate a random degree abbreviation.
        
        :example: 'BS'
        """
        return self.random_element(self.degree_abbrs)

    def degree(self) -> str:
        """
        Generate a full degree name.
        
        :example: 'Bachelor of Science in Computer Science'
        """
        degree_type = self.degree_type()
        field = self.field_of_study()
        
        if degree_type == "Associate":
            return f"Associate of Science in {field}"
        elif degree_type == "Bachelor":
            return f"Bachelor of Science in {field}"
        elif degree_type == "Master":
            return f"Master of Science in {field}"
        elif degree_type == "Doctorate":
            return f"Doctor of Philosophy in {field}"
        elif degree_type == "Certificate":
            return f"Certificate in {field}"
        else:
            return f"Diploma in {field}"

    def field_of_study(self) -> str:
        """
        Generate a random field of study.
        
        :example: 'Computer Science'
        """
        return self.random_element(self.fields_of_study)

    def university_name(self) -> str:
        """
        Generate a random university name.
        
        :example: 'Metropolitan State University'
        """
        prefix = self.generator.city()
        suffix = self.random_element(self.university_names)
        return f"{prefix} {suffix}"

    def school_level(self) -> str:
        """
        Generate a random school level.
        
        :example: 'High School'
        """
        return self.random_element(self.school_levels)

    def school_name(self) -> str:
        """
        Generate a random school name.
        
        :example: 'Lincoln High School'
        """
        name = self.generator.last_name()
        level = self.school_level()
        return f"{name} {level}"

    def academic_subject(self) -> str:
        """
        Generate a random academic subject.
        
        :example: 'Mathematics'
        """
        return self.random_element(self.academic_subjects)

    def course_name(self) -> str:
        """
        Generate a random course name.
        
        :example: 'Introduction to Computer Science'
        """
        prefix = self.random_element(self.course_prefixes)
        subject = self.academic_subject()
        return f"{prefix} {subject}"

    def course_code(self) -> str:
        """
        Generate a random course code.
        
        :example: 'CS-101'
        """
        dept = self.generator.lexify("???").upper()
        number = self.generator.random_int(100, 999)
        return f"{dept}-{number}"

    def academic_title(self) -> str:
        """
        Generate a random academic title.
        
        :example: 'Professor'
        """
        return self.random_element(self.academic_titles)

    def extracurricular_activity(self) -> str:
        """
        Generate a random extracurricular activity.
        
        :example: 'Debate Club'
        """
        return self.random_element(self.extracurricular_activities)

    def academic_honor(self) -> str:
        """
        Generate a random academic honor.
        
        :example: 'Dean\'s List'
        """
        return self.random_element(self.academic_honors)

    def letter_grade(self) -> str:
        """
        Generate a random letter grade.
        
        :example: 'A'
        """
        return self.random_element(self.letter_grades)

    def gpa(self, scale: str = "4.0") -> str:
        """
        Generate a random GPA.
        
        :param scale: GPA scale ('4.0', '5.0', or '100')
        :example: '3.75'
        """
        if scale == "4.0":
            gpa_value = round(self.generator.random.uniform(2.0, 4.0), 2)
            return f"{gpa_value:.2f}"
        elif scale == "5.0":
            gpa_value = round(self.generator.random.uniform(2.5, 5.0), 2)
            return f"{gpa_value:.2f}"
        elif scale == "100":
            gpa_value = self.generator.random_int(60, 100)
            return str(gpa_value)
        else:
            return self.gpa("4.0")

    def academic_term(self) -> str:
        """
        Generate a random academic term.
        
        :example: 'Fall Semester'
        """
        return self.random_element(self.academic_terms)

    def academic_year(self) -> str:
        """
        Generate a random academic year.
        
        :example: '2023-2024'
        """
        year = self.generator.random_int(2000, 2030)
        return f"{year}-{year + 1}"

    def campus_facility(self) -> str:
        """
        Generate a random campus facility.
        
        :example: 'Library'
        """
        return self.random_element(self.campus_facilities)

    def student_classification(self) -> str:
        """
        Generate a random student classification.
        
        :example: 'Junior'
        """
        return self.random_element(self.student_classifications)

    def student_id(self) -> str:
        """
        Generate a random student ID.
        
        :example: 'STU-2023-001234'
        """
        year = self.generator.random_int(2000, 2030)
        number = self.generator.random_int(100000, 999999)
        return f"STU-{year}-{number}"

    def academic_department(self) -> str:
        """
        Generate a random academic department.
        
        :example: 'Department of Computer Science'
        """
        return self.random_element(self.academic_departments)

    def credit_hours(self) -> int:
        """
        Generate random credit hours for a course.
        
        :example: 3
        """
        return self.random_element([1, 2, 3, 4, 5, 6])

    def graduation_year(self) -> int:
        """
        Generate a random graduation year.
        
        :example: 2024
        """
        return self.generator.random_int(1990, 2030)

    def class_size(self) -> int:
        """
        Generate a random class size.
        
        :example: 25
        """
        return self.generator.random_int(10, 300)

    def tuition_fee(self) -> str:
        """
        Generate a random tuition fee.
        
        :example: '$15,000'
        """
        fee = self.generator.random_int(5000, 60000)
        return f"${fee:,}"

    def scholarship_name(self) -> str:
        """
        Generate a random scholarship name.
        
        :example: 'Presidential Scholarship'
        """
        prefixes = (
            "Presidential",
            "Merit",
            "Academic Excellence",
            "Leadership",
            "Athletic",
            "Need-Based",
            "Diversity",
            "STEM",
            "Arts",
            "Community Service",
        )
        return f"{self.random_element(prefixes)} Scholarship"

    def scholarship_amount(self) -> str:
        """
        Generate a random scholarship amount.
        
        :example: '$5,000'
        """
        amount = self.generator.random_int(500, 50000)
        return f"${amount:,}"

    def attendance_rate(self) -> str:
        """
        Generate a random attendance rate.
        
        :example: '95%'
        """
        rate = self.generator.random_int(70, 100)
        return f"{rate}%"

    def study_hours(self) -> int:
        """
        Generate random study hours per week.
        
        :example: 15
        """
        return self.generator.random_int(5, 40)



from typing import List


def _digits_of(number: float) -> List[int]:
    return [int(digit) for digit in str(number)]


def luhn_checksum(number: float) -> int:
    digits = _digits_of(number)
    odd_digits = digits[-1::-2]
    even_digits = digits[-2::-2]

    checksum = sum(odd_digits) + sum(sum(_digits_of(digit * 2)) for digit in even_digits)

    return checksum % 10


def calculate_luhn(partial_number: float) -> int:
    """
    Generates the Checksum using Luhn's algorithm
    """
    check_digit = luhn_checksum(int(partial_number) * 10)
    return check_digit if check_digit == 0 else 10 - check_digit



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name}} {{last_name}}",
        "{{first_name_male}}-{{first_name_male}} {{last_name}}",
        "{{first_name_male}}-{{first_name_male}} {{last_name}}",
        "{{first_name_female}}-{{first_name_female}} {{last_name}}",
        "{{first_name_female}}-{{first_name_female}} {{last_name}}",
        "{{first_name}} {{last_name}}-{{last_name}}",
        "{{first_name}} {{last_name}}-{{last_name}}",
        "{{prefix}} {{first_name_male}} {{last_name}}",
    )

    # 100 most common male first names, alphabetically.
    # Source: http://www.ssb.no/a/navn/fornavn-menn-100.html
    first_names_male = (
        "Adrian",
        "Alexander",
        "Alf",
        "Anders",
        "Andreas",
        "Arild",
        "Arne",
        "Asbjørn",
        "Bjørn",
        "Christian",
        "Dag",
        "Daniel",
        "Egil",
        "Einar",
        "Eirik",
        "Eivind",
        "Emil",
        "Erik",
        "Erling",
        "Espen",
        "Finn",
        "Frank",
        "Fredrik",
        "Frode",
        "Geir",
        "Gunnar",
        "Hans",
        "Harald",
        "Helge",
        "Henrik",
        "Håkon",
        "Håvard",
        "Ivar",
        "Jan",
        "Jens",
        "Joakim",
        "Johannes",
        "Johan",
        "John",
        "Jonas",
        "Jon",
        "Jørgen",
        "Karl",
        "Kenneth",
        "Kim",
        "Kjell",
        "Kjetil",
        "Knut",
        "Kåre",
        "Kristian",
        "Kristoffer",
        "Lars",
        "Leif",
        "Magne",
        "Magnus",
        "Marius",
        "Markus",
        "Martin",
        "Mathias",
        "Morten",
        "Nils",
        "Odd",
        "Ola",
        "Olav",
        "Ole",
        "Per",
        "Petter",
        "Pål",
        "Roar",
        "Robert",
        "Roger",
        "Rolf",
        "Roy",
        "Rune",
        "Sander",
        "Sebastian",
        "Sigurd",
        "Simen",
        "Sindre",
        "Sondre",
        "Steinar",
        "Stein",
        "Stian",
        "Stig",
        "Svein",
        "Sverre",
        "Terje",
        "Thomas",
        "Thor",
        "Tobias",
        "Tommy",
        "Tom",
        "Torbjørn",
        "Tore",
        "Tor",
        "Trond",
        "Vegard",
        "Vidar",
        "Øystein",
        "Øyvind",
    )

    # 100 most common female first names, alphabetically.
    # Source: http://www.ssb.no/a/navn/fornavn-kvinner-100.html
    first_names_female = (
        "Andrea",
        "Anette",
        "Anita",
        "Anna",
        "Anne",
        "Ann",
        "Astrid",
        "Aud",
        "Bente",
        "Berit",
        "Bjørg",
        "Britt",
        "Camilla",
        "Cathrine",
        "Cecilie",
        "Elin",
        "Elisabeth",
        "Elise",
        "Eli",
        "Ellen",
        "Else",
        "Emilie",
        "Emma",
        "Eva",
        "Gerd",
        "Grete",
        "Grethe",
        "Gro",
        "Gunn",
        "Hanna",
        "Hanne",
        "Hege",
        "Heidi",
        "Helene",
        "Hilde",
        "Ida",
        "Ingeborg",
        "Inger",
        "Ingrid",
        "Irene",
        "Janne",
        "Jenny",
        "Jorunn",
        "Julie",
        "Karen",
        "Karin",
        "Kari",
        "Karoline",
        "Kirsten",
        "Kjersti",
        "Kristine",
        "Kristin",
        "Laila",
        "Lene",
        "Linda",
        "Line",
        "Linn",
        "Lise",
        "Liv",
        "Malin",
        "Maren",
        "Marianne",
        "Maria",
        "Marie",
        "Mari",
        "Marit",
        "Marte",
        "Martine",
        "May",
        "Mette",
        "Mona",
        "Monica",
        "Nina",
        "Nora",
        "Ragnhild",
        "Randi",
        "Reidun",
        "Rita",
        "Ruth",
        "Sara",
        "Sigrid",
        "Silje",
        "Siri",
        "Sissel",
        "Siv",
        "Sofie",
        "Solveig",
        "Stine",
        "Synnøve",
        "Thea",
        "Tone",
        "Tonje",
        "Torill",
        "Tove",
        "Trine",
        "Turid",
        "Unni",
        "Vilde",
        "Wenche",
        "Åse",
    )

    first_names = first_names_male + first_names_female

    # 100 most common last names, alphabetically.
    # Source: http://www.ssb.no/a/navn/alf/etter100.html
    last_names = (
        "Aasen",
        "Aas",
        "Abrahamsen",
        "Ahmed",
        "Ali",
        "Amundsen",
        "Andersen",
        "Andreassen",
        "Andresen",
        "Antonsen",
        "Arnesen",
        "Aune",
        "Bakken",
        "Bakke",
        "Berge",
        "Berg",
        "Berntsen",
        "Bøe",
        "Birkeland",
        "Brekke",
        "Christensen",
        "Dahl",
        "Danielsen",
        "Edvardsen",
        "Eide",
        "Eliassen",
        "Ellingsen",
        "Engen",
        "Eriksen",
        "Evensen",
        "Fredriksen",
        "Gulbrandsen",
        "Gundersen",
        "Hagen",
        "Halvorsen",
        "Hansen",
        "Hanssen",
        "Haugen",
        "Hauge",
        "Haugland",
        "Haug",
        "Helland",
        "Henriksen",
        "Holm",
        "Isaksen",
        "Iversen",
        "Jacobsen",
        "Jakobsen",
        "Jensen",
        "Jenssen",
        "Johannessen",
        "Johansen",
        "Johnsen",
        "Jørgensen",
        "Karlsen",
        "Knudsen",
        "Knutsen",
        "Kristensen",
        "Kristiansen",
        "Kristoffersen",
        "Larsen",
        "Lien",
        "Lie",
        "Lunde",
        "Lund",
        "Madsen",
        "Martinsen",
        "Mathisen",
        "Mikkelsen",
        "Moen",
        "Moe",
        "Myhre",
        "Myklebust",
        "Nguyen",
        "Nielsen",
        "Nilsen",
        "Næss",
        "Nygård",
        "Olsen",
        "Paulsen",
        "Pedersen",
        "Pettersen",
        "Rasmussen",
        "Rønning",
        "Ruud",
        "Sandvik",
        "Simonsen",
        "Sivertsen",
        "Solberg",
        "Solheim",
        "Sørensen",
        "Sæther",
        "Strand",
        "Strøm",
        "Svendsen",
        "Tangen",
        "Thomassen",
        "Thorsen",
        "Tveit",
        "Vik",
        "Ødegård",
    )

    prefixes = (
        "Dr.",
        "Prof.",
    )



from .. import BaseProvider

localized = False


class Provider(BaseProvider):
    """Provider for animal-related data."""

    # Mammals
    mammals = [
        "Dog", "Cat", "Lion", "Tiger", "Elephant", "Giraffe", "Zebra", "Horse",
        "Cow", "Pig", "Sheep", "Goat", "Rabbit", "Mouse", "Rat", "Hamster",
        "Guinea Pig", "Squirrel", "Chipmunk", "Beaver", "Otter", "Seal",
        "Walrus", "Whale", "Dolphin", "Porpoise", "Bear", "Panda", "Koala",
        "Kangaroo", "Wallaby", "Opossum", "Raccoon", "Skunk", "Fox", "Wolf",
        "Coyote", "Jackal", "Hyena", "Cheetah", "Leopard", "Jaguar", "Cougar",
        "Lynx", "Bobcat", "Monkey", "Gorilla", "Chimpanzee", "Orangutan",
        "Baboon", "Lemur", "Sloth", "Anteater", "Armadillo", "Hedgehog",
        "Porcupine", "Bat", "Mole", "Shrew", "Deer", "Moose", "Elk", "Caribou",
        "Antelope", "Gazelle", "Buffalo", "Bison", "Yak", "Camel", "Llama",
        "Alpaca", "Rhinoceros", "Hippopotamus", "Tapir", "Aardvark"
    ]

    # Birds
    birds = [
        "Eagle", "Hawk", "Falcon", "Owl", "Parrot", "Macaw", "Cockatoo",
        "Parakeet", "Canary", "Finch", "Sparrow", "Robin", "Blue Jay",
        "Cardinal", "Crow", "Raven", "Magpie", "Starling", "Swallow",
        "Swift", "Hummingbird", "Woodpecker", "Kingfisher", "Pelican",
        "Seagull", "Albatross", "Penguin", "Ostrich", "Emu", "Cassowary",
        "Kiwi", "Peacock", "Pheasant", "Quail", "Turkey", "Chicken",
        "Duck", "Goose", "Swan", "Flamingo", "Heron", "Stork", "Crane",
        "Ibis", "Spoonbill", "Vulture", "Condor", "Kite", "Buzzard"
    ]

    # Reptiles
    reptiles = [
        "Snake", "Python", "Boa", "Cobra", "Viper", "Rattlesnake", "Anaconda",
        "Lizard", "Gecko", "Iguana", "Chameleon", "Monitor Lizard",
        "Komodo Dragon", "Crocodile", "Alligator", "Caiman", "Gharial",
        "Turtle", "Tortoise", "Sea Turtle", "Terrapin", "Snapping Turtle"
    ]

    # Amphibians
    amphibians = [
        "Frog", "Toad", "Tree Frog", "Bullfrog", "Poison Dart Frog",
        "Salamander", "Newt", "Axolotl", "Caecilian"
    ]

    # Fish
    fishes = [
        "Goldfish", "Koi", "Betta", "Guppy", "Tetra", "Angelfish", "Discus",
        "Clownfish", "Tang", "Damselfish", "Wrasse", "Grouper", "Snapper",
        "Bass", "Trout", "Salmon", "Pike", "Catfish", "Carp", "Tuna",
        "Swordfish", "Marlin", "Shark", "Great White Shark", "Hammerhead Shark",
        "Tiger Shark", "Whale Shark", "Ray", "Manta Ray", "Stingray",
        "Eel", "Moray Eel", "Seahorse", "Pufferfish", "Lionfish", "Barracuda"
    ]

    # Invertebrates
    invertebrates = [
        "Butterfly", "Moth", "Bee", "Wasp", "Ant", "Termite", "Beetle",
        "Ladybug", "Firefly", "Dragonfly", "Damselfly", "Grasshopper",
        "Cricket", "Mantis", "Stick Insect", "Cockroach", "Fly", "Mosquito",
        "Spider", "Tarantula", "Scorpion", "Tick", "Mite", "Centipede",
        "Millipede", "Snail", "Slug", "Octopus", "Squid", "Cuttlefish",
        "Nautilus", "Jellyfish", "Coral", "Sea Anemone", "Starfish",
        "Sea Urchin", "Sea Cucumber", "Crab", "Lobster", "Shrimp", "Prawn",
        "Crayfish", "Barnacle", "Clam", "Oyster", "Mussel", "Scallop",
        "Worm", "Earthworm", "Leech"
    ]

    # Domestic animals
    domestic_animals = [
        "Dog", "Cat", "Horse", "Cow", "Pig", "Sheep", "Goat", "Chicken",
        "Duck", "Goose", "Turkey", "Rabbit", "Guinea Pig", "Hamster",
        "Gerbil", "Ferret", "Parrot", "Canary", "Goldfish"
    ]

    # Wild animals
    wild_animals = [
        "Lion", "Tiger", "Bear", "Wolf", "Elephant", "Giraffe", "Zebra",
        "Rhinoceros", "Hippopotamus", "Crocodile", "Alligator", "Shark",
        "Eagle", "Hawk", "Owl", "Snake", "Lizard", "Frog", "Whale", "Dolphin"
    ]

    # Endangered species
    endangered_species_list = [
        "Giant Panda", "Snow Leopard", "Tiger", "Asian Elephant",
        "Black Rhinoceros", "Orangutan", "Gorilla", "Blue Whale",
        "Sea Turtle", "Polar Bear", "Amur Leopard", "Vaquita",
        "Javan Rhinoceros", "Sumatran Elephant", "Hawksbill Turtle",
        "Saola", "Kakapo", "Philippine Eagle", "California Condor"
    ]

    # Dog breeds
    dog_breeds = [
        "Labrador Retriever", "German Shepherd", "Golden Retriever", "Bulldog",
        "Beagle", "Poodle", "Rottweiler", "Yorkshire Terrier", "Boxer",
        "Dachshund", "Siberian Husky", "Great Dane", "Doberman Pinscher",
        "Shih Tzu", "Boston Terrier", "Pomeranian", "Chihuahua", "Pug",
        "Border Collie", "Australian Shepherd", "Cocker Spaniel", "Maltese",
        "Shetland Sheepdog", "Cavalier King Charles Spaniel", "Corgi"
    ]

    # Cat breeds
    cat_breeds = [
        "Persian", "Maine Coon", "Siamese", "Ragdoll", "Bengal", "Abyssinian",
        "Birman", "Oriental Shorthair", "Sphynx", "Devon Rex", "American Shorthair",
        "British Shorthair", "Scottish Fold", "Russian Blue", "Norwegian Forest Cat",
        "Burmese", "Manx", "Himalayan", "Exotic Shorthair", "Turkish Angora"
    ]

    def animal(self) -> str:
        """
        Generate a random animal name from all categories.
        
        :example: 'Lion'
        """
        all_animals = (
            self.mammals + self.birds + self.reptiles + 
            self.amphibians + self.fishes + self.invertebrates
        )
        return self.random_element(all_animals)

    def mammal(self) -> str:
        """
        Generate a random mammal name.
        
        :example: 'Elephant'
        """
        return self.random_element(self.mammals)

    def bird(self) -> str:
        """
        Generate a random bird name.
        
        :example: 'Eagle'
        """
        return self.random_element(self.birds)

    def reptile(self) -> str:
        """
        Generate a random reptile name.
        
        :example: 'Snake'
        """
        return self.random_element(self.reptiles)

    def amphibian(self) -> str:
        """
        Generate a random amphibian name.
        
        :example: 'Frog'
        """
        return self.random_element(self.amphibians)

    def fish(self) -> str:
        """
        Generate a random fish name.
        
        :example: 'Goldfish'
        """
        return self.random_element(self.fishes)

    def invertebrate(self) -> str:
        """
        Generate a random invertebrate name.
        
        :example: 'Butterfly'
        """
        return self.random_element(self.invertebrates)

    def domestic_animal(self) -> str:
        """
        Generate a random domestic animal name.
        
        :example: 'Dog'
        """
        return self.random_element(self.domestic_animals)

    def wild_animal(self) -> str:
        """
        Generate a random wild animal name.
        
        :example: 'Lion'
        """
        return self.random_element(self.wild_animals)

    def endangered_species(self) -> str:
        """
        Generate a random endangered species name.
        
        :example: 'Giant Panda'
        """
        return self.random_element(self.endangered_species_list)

    def dog_breed(self) -> str:
        """
        Generate a random dog breed name.
        
        :example: 'Labrador Retriever'
        """
        return self.random_element(self.dog_breeds)

    def cat_breed(self) -> str:
        """
        Generate a random cat breed name.
        
        :example: 'Persian'
        """
        return self.random_element(self.cat_breeds)

    def animal_type(self) -> str:
        """
        Generate a random animal type/category.
        
        :example: 'Mammal'
        """
        types = ["Mammal", "Bird", "Reptile", "Amphibian", "Fish", "Invertebrate"]
        return self.random_element(types)



from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    # https://en.wikipedia.org/wiki/Telephone_numbers_in_Liechtenstein
    formats = ("%## ## ##", "+423 %## ## ##")



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{first_name_female}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{first_name_female}} {{last_name}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}} {{last_name}}",
    )

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{first_name_male}} {{last_name}}",
    )

    formats = formats_male + formats_female

    first_names_female = (
        "Abiye",
        "Acarkan",
        "Adal",
        "Adila",
        "Adviye",
        "Afife",
        "Ahter",
        "Akay",
        "Akgüneş",
        "Akise",
        "Akmaral",
        "Aksoy",
        "Akyıldız",
        "Alabezek",
        "Alaz",
        "Algış",
        "Alize",
        "Almast",
        "Alsoy",
        "Altınbike",
        "Altınçiçek",
        "Alışık",
        "Amre",
        "Anargül",
        "Anka",
        "Aral",
        "Armahan",
        "Arziye",
        "Arıpınar",
        "Asiman",
        "Asliye",
        "Asu",
        "Atanur",
        "Atiyye",
        "Avunç",
        "Ayasun",
        "Aybet",
        "Aycagül",
        "Aydar",
        "Ayduru",
        "Aygönenç",
        "Ayhan",
        "Aykut",
        "Aylil",
        "Aynilhayat",
        "Aynımah",
        "Aysema",
        "Aysevim",
        "Aysuna",
        "Ayten",
        "Aytöz",
        "Ayyaruk",
        "Ayçan",
        "Ayülker",
        "Ayşeana",
        "Ayşenur",
        "Azade",
        "Azize",
        "Açılay",
        "Ağbegim",
        "Aşhan",
        "Badegül",
        "Bahtinur",
        "Balca",
        "Ballı",
        "Banü",
        "Basriye",
        "Bağdat",
        "Bediriye",
        "Begim",
        "Behiza",
        "Belgizar",
        "Belkize",
        "Benek",
        "Benice",
        "Beray",
        "Bergen",
        "Beriye",
        "Berrin",
        "Besey",
        "Beste",
        "Beyhatun",
        "Bezek",
        "Bidayet",
        "Bilay",
        "Bilginur",
        "Bilkay",
        "Binay",
        "Birben",
        "Birgül",
        "Birsan",
        "Bitül",
        "Burcuhan",
        "Buşra",
        "Büreyre",
        "Büşranur",
        "Canan",
        "Canfeza",
        "Cannur",
        "Canseven",
        "Canur",
        "Cedide",
        "Cemiyle",
        "Cevale",
        "Ceyhun",
        "Cihan",
        "Cuheyna",
        "Damlanur",
        "Deha",
        "Deniz",
        "Deryanur",
        "Değer",
        "Dilara",
        "Dilcan",
        "Dilfeza",
        "Dilhuş",
        "Dilsitan",
        "Dilşat",
        "Divan",
        "Doğannur",
        "Duha",
        "Durgadin",
        "Dursadiye",
        "Duyguhan",
        "Döner",
        "Dürrüşehvar",
        "Ecegül",
        "Edaviye",
        "Efil",
        "Egenur",
        "Elamiye",
        "Elgin",
        "Elifnur",
        "Elvan",
        "Emal",
        "Emine.",
        "Emiş",
        "Enfes",
        "Erbay",
        "Erem",
        "Ergül",
        "Eriş",
        "Ervaniye",
        "Esengün",
        "Esmanperi",
        "Esna",
        "Evde",
        "Evrim",
        "Ezgin",
        "Eşim",
        "Fadile",
        "Fadıla",
        "Faize",
        "Fatigül",
        "Fatinur",
        "Fatmanur",
        "Fayize",
        "Feden",
        "Fehmiye",
        "Ferahdiba",
        "Feraye",
        "Ferhan",
        "Ferinaz",
        "Fermuta",
        "Feryas",
        "Feyha",
        "Feyzin",
        "Fidaye",
        "Fildan",
        "Firdevis",
        "Fitnat",
        "Fügen",
        "Gabel",
        "Ganiye",
        "Gelengül",
        "Gilman",
        "Goncafer",
        "Gök",
        "Gökperi",
        "Gökçe",
        "Göli",
        "Görsev",
        "Gözem",
        "Gül",
        "Gülay",
        "Gülbani",
        "Gülbeyan",
        "Gülbiye",
        "Gülcegün",
        "Güldam",
        "Gülder",
        "Güldünya",
        "Gülenay",
        "Güler",
        "Gülev",
        "Gülfari",
        "Gülfeza",
        "Gülgen",
        "Gülgüzel",
        "Gülhisar",
        "Gülinaz",
        "Gülkadın",
        "Güllühan",
        "Gülmisal",
        "Gülnaziye",
        "Gülper",
        "Gülsalın",
        "Gülselin",
        "Gülseren",
        "Gülsevil",
        "Gülsiye",
        "Gülsü",
        "Gülter",
        "Gülzadiye",
        "Gülçe",
        "Gülözge",
        "Gülüs",
        "Gülşa",
        "Gülşeref",
        "Günar",
        "Günebakan",
        "Güngören",
        "Günsel",
        "Günver",
        "Gürcüye",
        "Gürten",
        "Güverçin",
        "Güzey",
        "Habibe",
        "Hacile",
        "Hadrey",
        "Hafıza",
        "Halenur",
        "Haliye",
        "Hamiyet",
        "Hanbiken",
        "Hanim",
        "Hansultan",
        "Harbinaz",
        "Hasgül",
        "Hasret",
        "Hatin",
        "Havali",
        "Havse",
        "Hayel",
        "Hayrünnisa",
        "Hazine",
        "Hekime",
        "Henife",
        "Heva",
        "Hezniye",
        "Hilayda",
        "Hinet",
        "Hoşkadem",
        "Huban",
        "Hurican",
        "Hurşide",
        "Hüda",
        "Hümeyra",
        "Hürmet",
        "Hürüyet",
        "Hüsnühâl",
        "Ildız",
        "Irıs",
        "Işin",
        "Işın",
        "Jaruthip",
        "Kader",
        "Kadınana",
        "Kandef",
        "Kardelen",
        "Kaver",
        "Kefser",
        "Kerime",
        "Kezban",
        "Kifaye",
        "Kitan",
        "Koncagül",
        "Kumral",
        "Kutgün",
        "Kutun",
        "Kâzime",
        "Kübran",
        "Kısmet",
        "Laika",
        "Laze",
        "Lerze",
        "Leyli",
        "Lezize",
        "Limon",
        "Lâle",
        "Lüfen",
        "Macide",
        "Mahigül",
        "Mahnaz",
        "Mahter",
        "Maksüde",
        "Masume",
        "Maynur",
        "Maşide",
        "Mecide",
        "Mefharet",
        "Mehdiye",
        "Mehrigül",
        "Melaha",
        "Meleknur",
        "Melikkan",
        "Melûl",
        "Menfeat",
        "Menişan",
        "Merba",
        "Merim",
        "Merva",
        "Meryeme",
        "Mesude",
        "Meveddet",
        "Mevlüdiye",
        "Meyhanim",
        "Mezide",
        "Mihrab",
        "Mihriye",
        "Minibe",
        "Miray",
        "Misra",
        "Miyesser",
        "Muarra",
        "Mufide",
        "Muhiye",
        "Mujde",
        "Mukbile",
        "Musaffa",
        "Muvahhide",
        "Mübetcel",
        "Mücevher",
        "Müferrih",
        "Müjde",
        "Mükrüme",
        "Mümtaze",
        "Münezzer",
        "Müret",
        "Müsemma",
        "Müveddet",
        "Müğber",
        "Müşüre",
        "Nades",
        "Nafile",
        "Naide",
        "Nalân",
        "Narhanim",
        "Nasiba",
        "Natalia",
        "Naz",
        "Nazende",
        "Nazi",
        "Nazimet",
        "Nazlihan",
        "Nazıdil",
        "Nebiha",
        "Necilal",
        "Necva",
        "Nefaret",
        "Nefiye",
        "Nejdet",
        "Neptün",
        "Neriban",
        "Nesfe",
        "Neslinur",
        "Neval",
        "Nevgin",
        "Nevise",
        "Nevsale",
        "Neyran",
        "Nezengül",
        "Nezize",
        "Neşrin",
        "Nihan",
        "Nilcan",
        "Nili",
        "Nirgül",
        "Niğmet",
        "Nura",
        "Nurbanu",
        "Nurda",
        "Nurdeniz",
        "Nurey",
        "Nurgil",
        "Nurhayet",
        "Nuriyet",
        "Nurmelek",
        "Nurseda",
        "Nurser",
        "Nursim",
        "Nurtaç",
        "Nurveren",
        "Nurşan",
        "Nüdret",
        "Nürice",
        "Oguş",
        "Oluş",
        "Orçin",
        "Paksu",
        "Paye",
        "Pekkan",
        "Pembesin",
        "Peren",
        "Perinur",
        "Permun",
        "Pesent",
        "Piran",
        "Pürçek",
        "Rabbiye",
        "Rafia",
        "Rahiye",
        "Rakide",
        "Rana",
        "Rayla",
        "Rebihat",
        "Refet",
        "Rehime",
        "Rengül",
        "Revza",
        "Rezin",
        "Risalet",
        "Rojnu",
        "Ruhide",
        "Ruhugül",
        "Rumeysa",
        "Rümeysa",
        "Rıfkıye",
        "Sabihe",
        "Sabır",
        "Sadeti",
        "Sadiye",
        "Safinaz",
        "Safura",
        "Sahil",
        "Saire",
        "Salimet",
        "Samahat",
        "Sanavber",
        "Sanur",
        "Sarya",
        "Satıa",
        "Saygın",
        "Saçı",
        "Sebigül",
        "Seblâ",
        "Sedife",
        "Sefer",
        "Sehel",
        "Sejda",
        "Selcen",
        "Selime",
        "Selmin",
        "Selvi",
        "Selçuk",
        "Semat",
        "Semine",
        "Semrin",
        "Seniha",
        "Serda",
        "Serfinaz",
        "Serma",
        "Sernur",
        "Servinaz",
        "Sevcan",
        "Sevdinar",
        "Sevgen",
        "Sevginur",
        "Sevican",
        "Sevim",
        "Sevla",
        "Sevsevil",
        "Seyhan",
        "Seyyide",
        "Sezen",
        "Seçgül",
        "Sidar",
        "Silanur",
        "Simber",
        "Simten",
        "Sirap",
        "Siti",
        "Solma",
        "Sonnur",
        "Soykan",
        "Subutiye",
        "Sultane",
        "Sunay",
        "Susam",
        "Söyler",
        "Süheyda",
        "Süleyla",
        "Sümerya",
        "Süner",
        "Süsen",
        "Süzer",
        "Sırriye",
        "Tagangül",
        "Talibe",
        "Tan",
        "Tangül",
        "Tanses",
        "Tanyu",
        "Tasvir",
        "Tayyibe",
        "Taçnur",
        "Teknaz",
        "Temime",
        "Tercan",
        "Teybet",
        "Ticen",
        "Tomurcuk",
        "Tule",
        "Turcein",
        "Tutkucan",
        "Tuğçe",
        "Tülin",
        "Türcan",
        "Türknur",
        "Tüzenur",
        "Ufukay",
        "Ummahani",
        "Umuşan",
        "Uyanser",
        "Uğur",
        "Vacibe",
        "Varlık",
        "Vecide",
        "Vefia",
        "Verde",
        "Vezrife",
        "Vildane",
        "Yahşi",
        "Yalın",
        "Yasemen",
        "Yazgül",
        "Yaşar",
        "Yekbun",
        "Yepelek",
        "Yeşil",
        "Yosma",
        "Yurdaser",
        "Yurtseven",
        "Yücel",
        "Yıldız",
        "Zahfer",
        "Zaliha",
        "Zebirce",
        "Zehranur",
        "Zelha",
        "Zemzem",
        "Zerafet",
        "Zeride",
        "Zevlüde",
        "Zeyno",
        "Zilfa",
        "Zinnure",
        "Zubeyde",
        "Zöhrehan",
        "Züheyla",
        "Zülbiye",
        "Zülfüye",
        "Zümre",
        "Âlemşah",
        "Çalım",
        "Çağlar",
        "Çevregül",
        "Çilga",
        "Çisem",
        "Çolpan",
        "Ömriye",
        "Öncel",
        "Örfiye",
        "Övün",
        "Özay",
        "Özbilge",
        "Özdeş",
        "Özge",
        "Özgün",
        "Özlem",
        "Özpetek",
        "Özyurt",
        "Üge",
        "Ülke",
        "Ülküm",
        "Ümmahan",
        "Ümmühan",
        "Ümray",
        "Ünal",
        "Ünsever",
        "Ürper",
        "Üçgül",
        "İde",
        "İhsan",
        "İklim",
        "İldeş",
        "İlkbahar",
        "İlklima",
        "İlper",
        "İmge",
        "İmrihan",
        "İncifir",
        "İnsaf",
        "İrfan",
        "İslime",
        "İsra",
        "İzel",
        "İçimbike",
        "Şadıman",
        "Şahdiye",
        "Şahinder",
        "Şahnuray",
        "Şahıgül",
        "Şamiha",
        "Şayan",
        "Şazime",
        "Şefiye",
        "Şehreban",
        "Şehza",
        "Şelâle",
        "Şemsinisa",
        "Şendoğan",
        "Şennur",
        "Şeref",
        "Şerman",
        "Şevketfeza",
        "Şeyda",
        "Şilan",
        "Şirivan",
        "Şöhret",
        "Şüküfe",
    )

    first_names_male = (
        "Abdiş",
        "Abdulbekir",
        "Abdulgazi",
        "Abdulkadir",
        "Abdulmenaf",
        "Abdulsemet",
        "Abdurrahman",
        "Abdülahat",
        "Abdülcemal",
        "Abdülhadi",
        "Abdülkerim",
        "Abdülsamed",
        "Abdürreşit",
        "Abid",
        "Abuzar",
        "Acar",
        "Aclan",
        "Adak",
        "Adasal",
        "Adlan",
        "Adıgün",
        "Afer",
        "Ahat",
        "Ahsen",
        "Akalan",
        "Akar",
        "Akatay",
        "Akbaş",
        "Akboğa",
        "Akcivan",
        "Akdora",
        "Akdurmuş",
        "Akgöl",
        "Akif",
        "Akkerman",
        "Akmaner",
        "Aksay",
        "Aksöğüt",
        "Aktemür",
        "Akver",
        "Akçabay",
        "Akçasu",
        "Aköz",
        "Akınal",
        "Alaaddin",
        "Alaeddin",
        "Alanalp",
        "Alattin",
        "Alcan",
        "Alexandru",
        "Aliabbas",
        "Aliihsan",
        "Aliseydi",
        "Alkor",
        "Almus",
        "Alparslan",
        "Alpcan",
        "Alpin",
        "Alpsü",
        "Alsoy",
        "Altoğan",
        "Altuğ",
        "Altınkaya",
        "Altınışın",
        "Amaç",
        "Andiç",
        "Annak",
        "Apaydın",
        "Aran",
        "Arcan",
        "Argu",
        "Arifcan",
        "Arkut",
        "Arpağ",
        "Artan",
        "Aru",
        "Arıel",
        "Arıkol",
        "Arısoy",
        "Asalet",
        "Aslanhan",
        "Asım",
        "Atagün",
        "Atalay",
        "Atasagun",
        "Atatöre",
        "Atgun",
        "Atilhan",
        "Atnan",
        "Atılgan",
        "Avşin",
        "Ayaydın",
        "Aybora",
        "Aydinç",
        "Aydınbey",
        "Aygutalp",
        "Aykutalp",
        "Aypar",
        "Aysoy",
        "Aytek",
        "Aytuna",
        "Ayvas",
        "Ayşan",
        "Azettin",
        "Açıkel",
        "Ağakişi",
        "Ağmur",
        "Aşir",
        "Baba",
        "Bahaddin",
        "Bahittin",
        "Baki",
        "Balatekin",
        "Bali",
        "Baltaş",
        "Barak",
        "Bariş",
        "Barsen",
        "Barışcan",
        "Basım",
        "Baturay",
        "Batırhan",
        "Baydu",
        "Baykan",
        "Bayman",
        "Bayruhan",
        "Baytal",
        "Bayzettin",
        "Bağdaş",
        "Başay",
        "Başhan",
        "Başok",
        "Bedi",
        "Bedri",
        "Behrem",
        "Bekbay",
        "Bektaşi",
        "Bellisan",
        "Bengibay",
        "Benol",
        "Beren",
        "Berkal",
        "Berki",
        "Berksay",
        "Berran",
        "Besin",
        "Beyda",
        "Beyler",
        "Beyzade",
        "Bican",
        "Bilender",
        "Bilgen",
        "Bilgütay",
        "Biltaş",
        "Binbaşar",
        "Binışık",
        "Birgit",
        "Birsen",
        "Bişar",
        "Borahan",
        "Borataş",
        "Boynak",
        "Bozbağ",
        "Bozerk",
        "Boztaş",
        "Boğatimur",
        "Buhari",
        "Bulunç",
        "Burakhan",
        "Burç",
        "Buyrukhan",
        "Börteçin",
        "Büget",
        "Bünyamün",
        "Cabir",
        "Canal",
        "Canberk",
        "Candeniz",
        "Cangür",
        "Cannur",
        "Cansin",
        "Cantez",
        "Cavit",
        "Cebesoy",
        "Celilay",
        "Cemalettin",
        "Cenan",
        "Cercis",
        "Cevheri",
        "Cezayir",
        "Cihandide",
        "Cindoruk",
        "Coşkun",
        "Cuman",
        "Cüneyit",
        "Dalan",
        "Dalkılıç",
        "Danış",
        "Dayar",
        "Dağistan",
        "Delil",
        "Demirbüken",
        "Demiriz",
        "Demirok",
        "Demiryürek",
        "Denizalp",
        "Denkel",
        "Derkay",
        "Deviner",
        "Değmeer",
        "Diken",
        "Dilder",
        "Dincer",
        "Dinçkol",
        "Dinçsü",
        "Dirican",
        "Dirlik",
        "Dolun",
        "Dorukhan",
        "Doğanalp",
        "Doğanşah",
        "Doğuhan",
        "Duracan",
        "Durdali",
        "Durmuşali",
        "Duruk",
        "Duruöz",
        "Dölensoy",
        "Dündaralp",
        "Eba",
        "Ebuakil",
        "Ecemiş",
        "Edgübay",
        "Efe",
        "Eflâtun",
        "Efser",
        "Ekber",
        "Ekmel",
        "Elhan",
        "Elnur",
        "Elöve",
        "Emin",
        "Emirşan",
        "Emrullah",
        "Enes",
        "Enginiz",
        "Ensari",
        "Eral",
        "Eraycan",
        "Erbil",
        "Ercihan",
        "Erdemer",
        "Erdibay",
        "Erdogan",
        "Erdursun",
        "Erenalp",
        "Erensoy",
        "Ergener",
        "Erginel",
        "Ergönül",
        "Ergün",
        "Erik",
        "Erinçer",
        "Erkan",
        "Erkinel",
        "Erksoy",
        "Erkılıç",
        "Ermutlu",
        "Eroğul",
        "Ersel",
        "Erseç",
        "Ertan",
        "Erten",
        "Ertuncay",
        "Ertün",
        "Eryıldız",
        "Eröz",
        "Erşat",
        "Esenbay",
        "Esentürk",
        "Eskinalp",
        "Evcimen",
        "Evrimer",
        "Eyyüp",
        "Ezgütekin",
        "Eşref",
        "Fahrullah",
        "Fami",
        "Fatih",
        "Fazul",
        "Fehim",
        "Fenni",
        "Ferat",
        "Feremez",
        "Ferihan",
        "Fersan",
        "Ferzi",
        "Fetullah",
        "Feyruz",
        "Feza",
        "Filit",
        "Fuzuli",
        "Galip",
        "Gazel",
        "Gencaslan",
        "Gençay",
        "Geray",
        "Ginyas",
        "Giz",
        "Gökay",
        "Gökbudun",
        "Göken",
        "Göknur",
        "Göksev",
        "Gökten",
        "Gökçebalan",
        "Gökçil",
        "Gönen",
        "Görgünay",
        "Görklü",
        "Gözel",
        "Gücal",
        "Gülağa",
        "Gülel",
        "Güleğen",
        "Gülşahin",
        "Gümüştekin",
        "Günaydin",
        "Günden",
        "Gündüzalp",
        "Güneri",
        "Güngördü",
        "Günkurt",
        "Günser",
        "Günver",
        "Günşen",
        "Gürarda",
        "Gürelcem",
        "Gürsal",
        "Güçal",
        "Güçlüer",
        "Güçyeter",
        "Haciali",
        "Hakikat",
        "Halidun",
        "Haluk",
        "Hami",
        "Hanedan",
        "Hariz",
        "Hasbek",
        "Hatem",
        "Hayali",
        "Hayret",
        "Hazrat",
        "Hekmet",
        "Heyvetullah",
        "Hidir",
        "Hindal",
        "Hiçsönmez",
        "Hudavent",
        "Hunalp",
        "Huzuri",
        "Hükümdar",
        "Hürdoğan",
        "Hüryaşar",
        "Hüsmen",
        "Hıfzullah",
        "Idık",
        "Ilgı",
        "Ismık",
        "Işıkay",
        "Işıman",
        "Jankat",
        "Kader",
        "Kahir",
        "Kalgay",
        "Kamar",
        "Kanak",
        "Kanpulat",
        "Kapagan",
        "Karabaş",
        "Karaca",
        "Karaer",
        "Karakucak",
        "Karanbay",
        "Karataş",
        "Karcan",
        "Karlukhan",
        "Kasim",
        "Kavurt",
        "Kayagün",
        "Kaygusuz",
        "Kayrabay",
        "Kayıt",
        "Kaşif",
        "Kelâmi",
        "Kenter",
        "Kerman",
        "Kete",
        "Kibar",
        "Kiramettin",
        "Kiyasi",
        "Kocabaş",
        "Koldan",
        "Konguralp",
        "Kopan",
        "Koray",
        "Korkmazalp",
        "Korugan",
        "Kotuz",
        "Koçak",
        "Koçkan",
        "Koşukhan",
        "Kuddusi",
        "Kutay",
        "Kâmil",
        "Köker",
        "Köktaş",
        "Kösemen",
        "Kürşad",
        "Kılıçbay",
        "Kınel",
        "Kırat",
        "Kırgız",
        "Kıvılcım",
        "Kızıl",
        "Kızıltunç",
        "Ledün",
        "Lutfi",
        "Lütfi",
        "Mahir",
        "Mahsun",
        "Maksur",
        "Mansurali",
        "Masar",
        "Mazlum",
        "Mecit",
        "Mefarettin",
        "Mehmed",
        "Mehmetzahir",
        "Melihcan",
        "Memili",
        "Mengi",
        "Mengüç",
        "Merdi",
        "Mertel",
        "Merzuk",
        "Mestur",
        "Metinkaya",
        "Mevlüt",
        "Meşhur",
        "Mihin",
        "Milay",
        "Mirbadin",
        "Mishat",
        "Monis",
        "Mucahit",
        "Muhammet",
        "Muhip",
        "Muhyettin",
        "Muktedir",
        "Muratcan",
        "Musafet",
        "Mutasım",
        "Mutluhan",
        "Muvaffak",
        "Möhsim",
        "Mücellib",
        "Müfit",
        "Mükramin",
        "Mülâyim",
        "Münif",
        "Mürit",
        "Müslum",
        "Müzekker",
        "Nabil",
        "Nafii",
        "Nakip",
        "Nas",
        "Nasuf",
        "Nayil",
        "Nazlim",
        "Nebih",
        "Necdat",
        "Necmettin",
        "Nehip",
        "Nerim",
        "Nesip",
        "Nevsal",
        "Nezihi",
        "Nihai",
        "Niyazi",
        "Noman",
        "Nural",
        "Nurcan",
        "Nuretdin",
        "Nurkan",
        "Nurullah",
        "Nuyan",
        "N˜zamett˜n",
        "Odkanlı",
        "Oganer",
        "Okanay",
        "Okbay",
        "Okgüçlü",
        "Okseven",
        "Oktüremiş",
        "Okyalaz",
        "Olca",
        "Oldağ",
        "Oliver",
        "Omaca",
        "Onat",
        "Ongay",
        "Onuker",
        "Onurcan",
        "Onursu",
        "Oranlı",
        "Orgün",
        "Ortak",
        "Oruç",
        "Otay",
        "Oymak",
        "Ozansü",
        "Oğulbaş",
        "Oğurata",
        "Oğuzman",
        "Paker",
        "Pehlil",
        "Pirahmet",
        "Rabih",
        "Rafih",
        "Rahmet",
        "Ramadan",
        "Rasul",
        "Razı",
        "Recepali",
        "Refik",
        "Remazan",
        "Resulcan",
        "Rezzak",
        "Risalet",
        "Rohat",
        "Ruhsat",
        "Rüknettin",
        "Rüşen",
        "Saba",
        "Sabih",
        "Sadat",
        "Sadittin",
        "Safet",
        "Sahir",
        "Sakip",
        "Salami",
        "Salkın",
        "Salurbay",
        "Sami",
        "Samurtay",
        "Sancak",
        "Sançar",
        "Sargın",
        "Sarpkın",
        "Sarıcabay",
        "Satrettin",
        "Savak",
        "Savni",
        "Saydam",
        "Sayin",
        "Sayrak",
        "Sayın",
        "Sağcan",
        "Sağıt",
        "Sebattin",
        "Seda",
        "Seha",
        "Selaheddin",
        "Selatin",
        "Seler",
        "Selvi",
        "Selâtin",
        "Semender",
        "Sencar",
        "Seracettin",
        "Serda",
        "Serezli",
        "Serhatmehmet",
        "Serol",
        "Server",
        "Sevdi",
        "Sevindik",
        "Seydo",
        "Seyfullah",
        "Seyithan",
        "Sezal",
        "Sezginbaş",
        "Seçme",
        "Sidki",
        "Siper",
        "Sittik",
        "Sonad",
        "Songurkan",
        "Soydaner",
        "Soykut",
        "Soyselçuk",
        "Suat",
        "Sudi",
        "Sulhi",
        "Sunel",
        "Suphi",
        "Sökmen",
        "Sözer",
        "Sücaettin",
        "Süha",
        "Sümeyye",
        "Süvari",
        "Sıla",
        "Sıylıhan",
        "Taciddin",
        "Tahir",
        "Talayer",
        "Tali",
        "Tamaydın",
        "Tanak",
        "Tanbay",
        "Tandoğdu",
        "Tanhan",
        "Tanpınar",
        "Tansev",
        "Tansığ",
        "Tanyolaç",
        "Tanır",
        "Tarancı",
        "Tartış",
        "Tatu",
        "Tayaydın",
        "Taygan",
        "Taylak",
        "Tayyip",
        "Taşar",
        "Taşkan",
        "Teber",
        "Tecimer",
        "Tekbay",
        "Tekecan",
        "Tekiner",
        "Teksoy",
        "Telim",
        "Temirhan",
        "Temizkal",
        "Temuçin",
        "Tenvir",
        "Terlan",
        "Tevs",
        "Tezcan",
        "Tezol",
        "Timurtaş",
        "Tiğin",
        "Toker",
        "Toktuğ",
        "Toköz",
        "Tolonbay",
        "Tonguç",
        "Topuz",
        "Torhan",
        "Toy",
        "Toğan",
        "Tulun",
        "Tunahan",
        "Tunguç",
        "Tunçboğa",
        "Tunçkılıç",
        "Turabi",
        "Turgut",
        "Tutkun",
        "Tuyuğ",
        "Tuğcan",
        "Tuğrulhan",
        "Tuğtaş",
        "Törel",
        "Tükelalp",
        "Tümer",
        "Tümkurt",
        "Türabi",
        "Türkalp",
        "Türkmen",
        "Tüzeer",
        "Tınal",
        "Ufukay",
        "Ulakbey",
        "Ulu",
        "Uludağ",
        "Uluman",
        "Ulutay",
        "Uluğbey",
        "Umman",
        "Umutcan",
        "Uraltay",
        "Urhan",
        "Us",
        "Ushan",
        "Utkucan",
        "Uygun",
        "Uzbay",
        "Uzsoy",
        "Uçan",
        "Uçbeyi",
        "Uğan",
        "Uğurkan",
        "Uğurtan",
        "Vafir",
        "Vahittin",
        "Vargın",
        "Vaysal",
        "Vedat",
        "Veis",
        "Velitdin",
        "Verim",
        "Vezat",
        "Vâlâ",
        "Yadigar",
        "Yahşikan",
        "Yalazabay",
        "Yalgın",
        "Yaltırak",
        "Yalın",
        "Yamin",
        "Yankı",
        "Yargı",
        "Yasan",
        "Yavuz",
        "Yayak",
        "Yazganalp",
        "Yağın",
        "Yağızkurt",
        "Yaşattin",
        "Yekda",
        "Yelesen",
        "Yeneral",
        "Yertan",
        "Yetişal",
        "Yigit",
        "Yilmaz",
        "Yolal",
        "Yoruç",
        "Yunt",
        "Yurdanur",
        "Yurtgüven",
        "Yurttaş",
        "Yönetmen",
        "Yücelen",
        "Yümun",
        "Yıldır",
        "Yılma",
        "Zahid",
        "Zamir",
        "Zekayi",
        "Zennun",
        "Zeynelabidin",
        "Zihni",
        "Ziyaettin",
        "Zoral",
        "Züfer",
        "Zülgarni",
        "Âdem",
        "Çakar",
        "Çakırca",
        "Çaltı",
        "Çamok",
        "Çapkan",
        "Çavuldur",
        "Çağa",
        "Çağdan",
        "Çağlasın",
        "Çağveren",
        "Çelem",
        "Çelikkan",
        "Çelikyürek",
        "Çerçi",
        "Çetinsu",
        "Çeviköz",
        "Çinerk",
        "Çokan",
        "Çopur",
        "Çoğay",
        "Çıdal",
        "Çıvgın",
        "Öge",
        "Ökkaş",
        "Öktürk",
        "Ömür",
        "Öncel",
        "Önel",
        "Öngen",
        "Önsal",
        "Örik",
        "Öryürek",
        "Över",
        "Özakan",
        "Özalpsan",
        "Özaslan",
        "Özbay",
        "Özbilek",
        "Özdal",
        "Özdil",
        "Özdoğdu",
        "Özel",
        "Özerdinç",
        "Özertem",
        "Özger",
        "Özgür",
        "Özinal",
        "Özkent",
        "Özkutlu",
        "Özlü",
        "Özokçu",
        "Özpınar",
        "Özsözlü",
        "Öztek",
        "Öztürk",
        "Özçam",
        "Özüdoğru",
        "Öğet",
        "Übeydullah",
        "Ülfet",
        "Ülküdeş",
        "Ümmet",
        "Ünek",
        "Ünlen",
        "Ünsever",
        "Ünübol",
        "Ürfettin",
        "Üsame",
        "Üstün",
        "Üzer",
        "Ğanim",
        "İbrahim",
        "İdiris",
        "İkiz",
        "İlalmış",
        "İlbek",
        "İldem",
        "İlgi",
        "İlim",
        "İlkim",
        "İlmafer",
        "İlsu",
        "İlteriş",
        "İmam",
        "İmren",
        "İnançlı",
        "İntihap",
        "İsak",
        "İsmet",
        "İvecen",
        "İyiyürek",
        "İzgü",
        "İşcan",
        "Şabettin",
        "Şafii",
        "Şahat",
        "Şahinbey",
        "Şahmettin",
        "Şali",
        "Şanlı",
        "Şavki",
        "Şefi",
        "Şehamet",
        "Şekim",
        "Şemsettin",
        "Şendoğan",
        "Şenkal",
        "Şerafeddin",
        "Şevket",
        "Şide",
        "Şinasi",
        "Şuayp",
        "Şükri",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Yılmaz",
        "Yıldırım",
        "Türk",
        "Yorulmaz",
        "Erdoğan",
        "Çorlu",
        "Sakarya",
        "Demir",
        "Yaman",
        "Manço",
        "Aksu",
        "Akçay",
        "Akar",
        "Bilir",
        "Bilgin",
        "Yüksel",
        "Eraslan",
        "Aslan",
        "Arslan",
        "Hançer",
        "Hayrioğlu",
        "Şama",
        "Ergül",
        "Arsoy",
        "Kısakürek",
        "Gülen",
        "Seven",
        "Şafak",
        "Dumanlı",
        "Ertaş",
        "Güçlü",
        "Soylu",
        "Zorlu",
        "Fırat",
        "Duran",
        "Durmuş",
        "Durdu",
        "Zengin",
        "Tevetoğlu",
        "Mansız",
        "Şener",
        "Şensoy",
        "Ülker",
        "Tarhan",
        "Sezer",
        "Demirel",
        "Gül",
        "Korutürk",
        "İnönü",
        "Öcalan",
        "Çetin",
        "Sezgin",
        "Alemdar",
        "Karadeniz",
        "Akdeniz",
        "Bilge",
        "Akgündüz",
        "Akçay",
        "Çamurcuoğlu",
        "İhsanoğlu",
        "Akça",
    )

    prefixes_female = (
        "Bayan",
        "Dr.",
        "Uz.",
        "Öğr.",
        "Çev.",
        "Okt.",
        "Öğr.",
        "Öğr.",
        "Arş. Gör.",
        "Yrd. Doç.",
        "Dr.",
        "Doç.",
        "Prof.",
        "Av.",
    )
    prefixes_male = (
        "Bay",
        "Dr.",
        "Uz.",
        "Öğr.",
        "Çev.",
        "Okt.",
        "Öğr.",
        "Öğr.",
        "Arş. Gör.",
        "Yrd. Doç.",
        "Dr.",
        "Doç.",
        "Prof.",
        "Av.",
    )

    prefixes = prefixes_female + prefixes_male



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``uz_UZ`` locale."""

    # Source: https://uz.wiktionary.org/wiki/Vikilug%E2%80%98at:Ranglar
    all_colors = OrderedDict(
        (
            ("Akvamarin", "#7FFFD4"),
            ("Anor", "#800000"),
            ("Apelsin", "#FFA000"),
            ("Bej", "#F5F5DC"),
            ("Binafsha", "#8B00FF"),
            ("Bodom", "#FFEBCD"),
            ("Bordo rang", "#800000"),
            ("Doimiy sariq", "#FFBF00"),
            ("Hantal", "#120A8F"),
            ("Havo rang", "#000080"),
            ("Indigo", "#4B0082"),
            ("Jigar rang", "#964B00"),
            ("Kul", "#808080"),
            ("Kumush", "#C0C0C0"),
            ("Koʻk", "#0000FF"),
            ("Kremi", "#FFFDD0"),
            ("Magenta", "#FF00FF"),
            ("Malina", "#DC143C"),
            ("Marjon", "#FF7F50"),
            ("Moshrang", "#C3B091"),
            ("Oq", "#FFFFFF"),
            ("Oxra", "#CC7722"),
            ("Oltin", "#FFD700"),
            ("Pushti", "#FFC0CB"),
            ("Qizil", "#FF0000"),
            ("Qizgʻish binafsharang", "#E0B0FF"),
            ("Qora", "#000000"),
            ("Qizil-sariq", "#FF8C69"),
            ("Samoviy", "#87CEFF"),
            ("Sariq", "#FFFF00"),
            ("Siyohrang", "#660099"),
            ("Sepya", "#705714"),
            ("Siena", "#FF8247"),
            ("Suv", "#00FFFF"),
            ("Terrakota", "#E2725B"),
            ("Turkuaz", "#30D5C8"),
            ("Ultramarin", "#120A8F"),
            ("Yashil", "#00FF00"),
            ("Zumrad", "#50C878"),
        )
    )

    safe_colors = (
        "Oq",
        "Qora",
        "Yashil",
        "Ko'k",
        "Qizil",
        "Sariq",
        "Pushti",
        "Olov",
        "Qaymoq",
        "Laym",
        "Kumush",
        "Kulrang",
    )



from randum.providers import BaseProvider


class Provider(BaseProvider):
    """
    Provider for Digital Object Identifier (DOI)
    Source of info: https://en.wikipedia.org/wiki/Digital_object_identifier (English)
    """

    def doi(self) -> str:
        """
        Generate a valid Digital Object Identifier (DOI).
        Format: 10.{4-9 digits}/{alphanumeric string}
        Eg: 10.1000/xyz123

        :sample:
        """
        prefix = "10"
        registrant = str(self.generator.random.randint(1000, 99999999))
        suffix = self.generator.bothify("?#?#?##").lower()

        return f"{prefix}.{registrant}/{suffix}"
